@@ -1,4 +1,4 @@
-/* $Id: softhsm.cpp 3911 2010-09-09 11:22:35Z rb $ */
+/* $Id: softhsm.cpp 5304 2011-07-08 11:07:06Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -48,10 +48,15 @@
 #include <iostream>
 #include <fstream>
 #include <sched.h>
-#if defined(HAVE_DLOPEN)
+
+#ifdef HAVE_DLOPEN
 #include <dlfcn.h>
 #endif
 
+#ifdef HAVE_LOADLIBRARY
+#include <windows.h>
+#endif
+
 // Includes for the crypto library
 #include <botan/auto_rng.h>
 #include <botan/rsa.h>
@@ -60,7 +65,6 @@
 #include <botan/init.h>
 #include <botan/libstate.h>
 #include <botan/numthry.h>
-using namespace Botan;
 
 void usage() {
   printf("Support tool for libsofthsm\n");
@@ -86,6 +90,8 @@ void usage() {
   printf("                    Use with --slot and --pin.\n");
   printf("                    WARNING: Make sure that no application is currently\n");
   printf("                    using SoftHSM and session objects.\n");
+  printf("  --trusted <bool>  Mark the object as trusted. true or false.\n");
+  printf("                    Use with --slot, --so-pin, --type, and (--id or --label).\n");
   printf("  --file-pin <PIN>  Supply a PIN if the file is encrypted.\n");
   printf("  --force           Override some warnings.\n");
   printf("  -h                Shows this help screen.\n");
@@ -98,6 +104,7 @@ void usage() {
   printf("  --pin <PIN>       The PIN for the normal user.\n");
   printf("  --slot <number>   The slot where the token is located.\n");
   printf("  --so-pin <PIN>    The PIN for the Security Officer (SO).\n");
+  printf("  --type <text>     The type of object. CKO_PUBLIC_KEY or CKO_CERTIFICATE.\n");
   printf("  -v                Show version info.\n");
   printf("  --version         Show version info.\n");
   printf("\n");
@@ -122,11 +129,13 @@ enum {
   OPT_IMPORT,
   OPT_EXPORT,
   OPT_OPTIMIZE,
+  OPT_TRUSTED,
   OPT_SLOT,
   OPT_LABEL,
   OPT_MODULE,
   OPT_ID,
   OPT_SO_PIN,
+  OPT_TYPE,
   OPT_PIN,
   OPT_FILE_PIN,
   OPT_FORCE,
@@ -140,11 +149,13 @@ static const struct option long_options[] = {
   { "import",          1, NULL, OPT_IMPORT },
   { "export",          1, NULL, OPT_EXPORT },
   { "optimize",        0, NULL, OPT_OPTIMIZE },
+  { "trusted",         1, NULL, OPT_TRUSTED },
   { "slot",            1, NULL, OPT_SLOT },
   { "label",           1, NULL, OPT_LABEL },
   { "module",          1, NULL, OPT_MODULE },
   { "id",              1, NULL, OPT_ID },
   { "so-pin",          1, NULL, OPT_SO_PIN },
+  { "type",            1, NULL, OPT_TYPE },
   { "pin",             1, NULL, OPT_PIN },
   { "file-pin",        1, NULL, OPT_FILE_PIN },
   { "force",           0, NULL, OPT_FORCE },
@@ -153,6 +164,25 @@ static const struct option long_options[] = {
   { NULL,              0, NULL, 0 }
 };
 
+#ifdef WIN32
+#include <conio.h>
+char *getpass(const char *prompt) {
+  static char buf[MAX_PIN_LEN+1];
+  size_t i;
+
+  fputs(prompt, stderr);
+  fflush(stderr);
+  for(i = 0; i < sizeof(buf) - 1; i++) {
+    buf[i] = _getch();
+    if(buf[i] == '\r')
+      break;
+  }
+  buf[i] = 0;
+  fputs("\n", stderr);
+  return buf;
+}
+#endif
+
 int main(int argc, char *argv[]) {
   int option_index = 0;
   int opt;
@@ -160,6 +190,8 @@ int main(int argc, char *argv[]) {
   char *inPath = NULL;
   char *outPath = NULL;
   char *soPIN = NULL;
+  char *boolTrusted = NULL;
+  char *type = NULL;
   char *userPIN = NULL;
   char *filePIN = NULL;
   char *label = NULL;
@@ -173,6 +205,7 @@ int main(int argc, char *argv[]) {
   int doImport = 0;
   int doExport = 0;
   int doOptimize = 0;
+  int doTrusted = 0;
   int action = 0;
 
   moduleHandle = NULL;
@@ -203,6 +236,11 @@ int main(int argc, char *argv[]) {
         doOptimize = 1;
         action++;
         break;
+      case OPT_TRUSTED:
+        doTrusted = 1;
+        action++;
+        boolTrusted = optarg;
+        break;
       case OPT_SLOT:
         slot = optarg;
         break;
@@ -218,6 +256,9 @@ int main(int argc, char *argv[]) {
       case OPT_SO_PIN:
         soPIN = optarg;
         break;
+      case OPT_TYPE:
+        type = optarg;
+        break;
       case OPT_PIN:
         userPIN = optarg;
         break;
@@ -262,16 +303,20 @@ int main(int argc, char *argv[]) {
 
   // The PKCS#11 library might be using Botan
   // Check if it has already initialized Botan
-  Library_State* state = swap_global_state(0);
-  // now put it back
-  swap_global_state(state);
+#ifdef BOTAN_PRE_1_9_10_FIX
+  Botan::Library_State* state = Botan::swap_global_state(0);
+  Botan::swap_global_state(state);
+#else
+  Botan::Library_State* state = Botan::Global_State_Management::swap_global_state(0);
+  Botan::Global_State_Management::swap_global_state(state);
+#endif
 
   if(state) {
     was_initialized = true;
   }
 
   if(was_initialized == false) {
-    LibraryInitializer::initialize("thread_safe=true");
+    Botan::LibraryInitializer::initialize("thread_safe=true");
   }
 
   // We should create the token.
@@ -307,15 +352,20 @@ int main(int argc, char *argv[]) {
     }
   }
 
+  // Set CKA_TRUSTED
+  if(doTrusted) {
+    trustObject(boolTrusted, slot, soPIN, type, label, objectID);
+  }
+
   if(was_initialized == false) {
-    LibraryInitializer::deinitialize();
+    Botan::LibraryInitializer::deinitialize();
   }
 
   if(action) {
     p11->C_Finalize(NULL_PTR);
     if(moduleHandle) {
 #if defined(HAVE_LOADLIBRARY)
-      // no idea
+      FreeLibrary((HMODULE)moduleHandle);
 #elif defined(HAVE_DLOPEN)
       dlclose(moduleHandle);
 #endif
@@ -331,20 +381,24 @@ CK_C_GetFunctionList loadLibrary(char *module) {
 
 #if defined(HAVE_LOADLIBRARY)
   // Load PKCS #11 library
+  HMODULE pDynLib = NULL;
   if(module) {
-    HINSTANCE hDLL = LoadLibrary(_T(module));
+    pDynLib = LoadLibrary(module);
   } else {
-    HINSTANCE hDLL = LoadLibrary(_T(DEFAULT_PKCS11_LIB));
+    pDynLib = LoadLibrary(DEFAULT_PKCS11_LIB);
   }
 
-  if(hDLL == NULL) {
+  if(pDynLib == NULL) {
     // Failed to load the PKCS #11 library
     return NULL;
   }
 
   // Retrieve the entry point for C_GetFunctionList
-  pGetFunctionList = (CK_C_GetFunctionList) GetProcAddress(hDLL, _T("C_GetFunctionList"));
-            
+  pGetFunctionList = (CK_C_GetFunctionList) GetProcAddress(pDynLib, "C_GetFunctionList");
+
+  // Store the handle so we can close it later
+  moduleHandle = pDynLib;
+
 #elif defined(HAVE_DLOPEN)
   // Load PKCS #11 library
   void* pDynLib;
@@ -605,7 +659,7 @@ void importKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, cha
     return;
   }
 
-  CK_OBJECT_HANDLE oHandle = searchObject(hSession, objID, objIDLen);
+  CK_OBJECT_HANDLE oHandle = searchObject(hSession, CKO_PRIVATE_KEY, NULL, objID, objIDLen);
   if(oHandle != CK_INVALID_HANDLE && forceExec == 0) {
     free(objID);
     fprintf(stderr, "Error: The ID is already assigned to another object. Use --force to override this message.\n");
@@ -733,7 +787,7 @@ void exportKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, cha
   }
 
   // Find the object handle
-  CK_OBJECT_HANDLE oHandle = searchObject(hSession, objID, objIDLen);
+  CK_OBJECT_HANDLE oHandle = searchObject(hSession, CKO_PRIVATE_KEY, NULL, objID, objIDLen);
   free(objID);
   if(oHandle == CK_INVALID_HANDLE) {
     fprintf(stderr, "Error: Could not find the private key with ID = %s\n", objectID);
@@ -747,7 +801,7 @@ void exportKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, cha
   }
 
   // Extract the key directly from the database
-  Private_Key *privKey = getPrivKey(dbPath, oHandle);
+  Botan::Private_Key *privKey = getPrivKey(dbPath, oHandle);
   free(dbPath);
   if(privKey == NULL) {
     return;
@@ -808,6 +862,125 @@ void optimize(char *slot, char *userPIN) {
   free(dbPath);
 }
 
+
+void trustObject(char *boolTrusted, char *slot, char *soPIN, char *type, char *label, char *objectID) {
+  char so_pin_copy[MAX_PIN_LEN+1];
+  CK_BBOOL trusted;
+  CK_OBJECT_CLASS oClass;
+
+  if(boolTrusted == NULL) {
+    fprintf(stderr, "Error: A boolean value must be supplied. Use --trusted <bool>\n");
+    return;
+  }
+  if(strncasecmp(boolTrusted, "true", 4) == 0) {
+    trusted = CK_TRUE;
+  } else if(strncasecmp(boolTrusted, "false", 5) == 0) {
+    trusted = CK_FALSE;
+  } else {
+    fprintf(stderr, "Error: Please use true or false as an input for --trusted <bool>\n");
+    return;
+  }
+
+  if(slot == NULL) {
+    fprintf(stderr, "Error: A slot number must be supplied. Use --slot <number>\n");
+    return;
+  }
+
+  if(objectID == NULL && label == NULL) {
+    fprintf(stderr, "Error: An ID or label for the object must be supplied. Use --id <hex> or --label <text>\n");
+    return;
+  }
+  int objIDLen = 0;
+  char *objID = NULL;
+  if(objectID != NULL) {
+    objID = hexStrToBin(objectID, strlen(objectID), &objIDLen);
+    if(objID == NULL) {
+      fprintf(stderr, "Please edit --id <hex> to correct error.\n");
+      return;
+    }
+  }
+
+  if(type == NULL) {
+    fprintf(stderr, "Error: An object type must must be supplied. Use --type <text>\n");
+    if(objID) free(objID);
+    return;
+  }
+  if(strncasecmp(type, "CKO_CERTIFICATE", 15) == 0) {
+    oClass = CKO_CERTIFICATE;
+  } else if(strncasecmp(type, "CKO_PUBLIC_KEY", 14) == 0) {
+    oClass = CKO_PUBLIC_KEY;
+  } else {
+    fprintf(stderr, "Error: Please use CKO_CERTIFICATE or CKO_PUBLIC_KEY as an input for --type <text>\n");
+    if(objID) free(objID);
+    return;
+  }
+
+  if(soPIN == NULL) {
+    printf("The SO PIN must have a length between %i and %i characters.\n", MIN_PIN_LEN, MAX_PIN_LEN);
+    #ifdef HAVE_GETPASSPHRASE
+      soPIN = getpassphrase("Enter SO PIN: ");
+    #else
+      soPIN = getpass("Enter SO PIN: ");
+    #endif
+  }
+
+  int soLength = strlen(soPIN);
+  while(soLength < MIN_PIN_LEN || soLength > MAX_PIN_LEN) {
+    printf("Wrong size! The SO PIN must have a length between %i and %i characters.\n", MIN_PIN_LEN, MAX_PIN_LEN);
+    #ifdef HAVE_GETPASSPHRASE
+      soPIN = getpassphrase("Enter SO PIN: ");
+    #else
+      soPIN = getpass("Enter SO PIN: ");
+    #endif
+    soLength = strlen(soPIN);
+  }
+  strcpy(so_pin_copy, soPIN);
+
+  CK_SLOT_ID slotID = atoi(slot);
+  CK_SESSION_HANDLE hSession;
+  CK_RV rv = p11->C_OpenSession(slotID, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSession);
+  if(rv != CKR_OK) {
+    if(rv == CKR_SLOT_ID_INVALID) {
+      fprintf(stderr, "Error: The given slot does not exist.\n");
+    } else {
+      fprintf(stderr, "Error: Could not open a session on the given slot.\n");
+    }
+    if(objID) free(objID);
+    return;
+  }
+
+  rv = p11->C_Login(hSession, CKU_SO, (CK_UTF8CHAR_PTR)so_pin_copy, soLength);
+  if(rv != CKR_OK) {
+    if(rv == CKR_PIN_INCORRECT) {
+      fprintf(stderr, "Error: The given user PIN does not match the one in the token.\n");
+    } else {
+      fprintf(stderr, "Error: Could not log in on the token.\n");
+    }
+    if(objID) free(objID);
+    return;
+  }
+
+  // Find object
+  CK_OBJECT_HANDLE oHandle = searchObject(hSession, oClass, label, objID, objIDLen);
+  if(objID) free(objID);
+  if(oHandle == CK_INVALID_HANDLE) {
+    fprintf(stderr, "Error: Could not find a matching object. The SO can only see public objects.\n");
+    return;
+  }
+
+  // Set value
+  CK_ATTRIBUTE objTemplate[] = {
+    { CKA_TRUSTED, &trusted, sizeof(trusted) }
+  };
+  rv = p11->C_SetAttributeValue(hSession, oHandle, objTemplate, 1);
+  if(rv != CKR_OK) {
+    fprintf(stderr, "Error: Could not modify CKA_TRUSTED. rv = 0x%08X\n", rv);
+    return;
+  }
+
+  printf("The CKA_TRUSTED has been modified.\n");  
+}
+
 int removeSessionObjs(char *dbPath) {
   sqlite3 *db = NULL;
   const char select_str[] = "SELECT objectID FROM Attributes WHERE type = ? AND value = ?;";
@@ -941,14 +1114,14 @@ key_material_t* importKeyMat(char *filePath, char *filePIN) {
     return NULL;
   }
 
-  AutoSeeded_RNG *rng = new AutoSeeded_RNG();
-  Private_Key *privKey = NULL;
+  Botan::AutoSeeded_RNG *rng = new Botan::AutoSeeded_RNG();
+  Botan::Private_Key *privKey = NULL;
 
   try {
     if(filePIN == NULL) {
-      privKey = PKCS8::load_key(filePath, *rng);
+      privKey = Botan::PKCS8::load_key(filePath, *rng);
     } else {
-      privKey = PKCS8::load_key(filePath, *rng, filePIN);
+      privKey = Botan::PKCS8::load_key(filePath, *rng, filePIN);
     }
   }
   catch(std::exception& e) {
@@ -965,10 +1138,10 @@ key_material_t* importKeyMat(char *filePath, char *filePIN) {
     return NULL;
   }
 
-  IF_Scheme_PrivateKey *ifKeyPriv = dynamic_cast<IF_Scheme_PrivateKey*>(privKey);
-  BigInt d1 = ifKeyPriv->get_d() % (ifKeyPriv->get_p() - 1);
-  BigInt d2 = ifKeyPriv->get_d() % (ifKeyPriv->get_q() - 1);
-  BigInt c = inverse_mod(ifKeyPriv->get_q(), ifKeyPriv->get_p());
+  Botan::IF_Scheme_PrivateKey *ifKeyPriv = dynamic_cast<Botan::IF_Scheme_PrivateKey*>(privKey);
+  Botan::BigInt d1 = ifKeyPriv->get_d() % (ifKeyPriv->get_p() - 1);
+  Botan::BigInt d2 = ifKeyPriv->get_d() % (ifKeyPriv->get_q() - 1);
+  Botan::BigInt c = inverse_mod(ifKeyPriv->get_q(), ifKeyPriv->get_p());
   key_material_t *keyMat = (key_material_t *)malloc(sizeof(key_material_t));
   keyMat->sizeE = ifKeyPriv->get_e().bytes();
   keyMat->sizeN = ifKeyPriv->get_n().bytes();
@@ -986,14 +1159,14 @@ key_material_t* importKeyMat(char *filePath, char *filePIN) {
   keyMat->bigDMP1 = (CK_VOID_PTR)malloc(keyMat->sizeDMP1);
   keyMat->bigDMQ1 = (CK_VOID_PTR)malloc(keyMat->sizeDMQ1);
   keyMat->bigIQMP = (CK_VOID_PTR)malloc(keyMat->sizeIQMP);
-  ifKeyPriv->get_e().binary_encode((byte *)keyMat->bigE);
-  ifKeyPriv->get_n().binary_encode((byte *)keyMat->bigN);
-  ifKeyPriv->get_d().binary_encode((byte *)keyMat->bigD);
-  ifKeyPriv->get_p().binary_encode((byte *)keyMat->bigP);
-  ifKeyPriv->get_q().binary_encode((byte *)keyMat->bigQ);
-  d1.binary_encode((byte *)keyMat->bigDMP1);
-  d2.binary_encode((byte *)keyMat->bigDMQ1);
-  c.binary_encode((byte *)keyMat->bigIQMP);
+  ifKeyPriv->get_e().binary_encode((Botan::byte *)keyMat->bigE);
+  ifKeyPriv->get_n().binary_encode((Botan::byte *)keyMat->bigN);
+  ifKeyPriv->get_d().binary_encode((Botan::byte *)keyMat->bigD);
+  ifKeyPriv->get_p().binary_encode((Botan::byte *)keyMat->bigP);
+  ifKeyPriv->get_q().binary_encode((Botan::byte *)keyMat->bigQ);
+  d1.binary_encode((Botan::byte *)keyMat->bigDMP1);
+  d2.binary_encode((Botan::byte *)keyMat->bigDMQ1);
+  c.binary_encode((Botan::byte *)keyMat->bigIQMP);
   delete privKey;
 
   return keyMat;
@@ -1034,20 +1207,30 @@ void freeKeyMaterial(key_material_t *keyMaterial) {
 
 // Search for an object
 
-CK_OBJECT_HANDLE searchObject(CK_SESSION_HANDLE hSession, char *objID, int objIDLen) {
-  if(objID == NULL) {
+CK_OBJECT_HANDLE searchObject(CK_SESSION_HANDLE hSession, CK_OBJECT_CLASS oClass, char *label, char *objID, int objIDLen) {
+  if(objID == NULL && label == NULL ) {
     return CK_INVALID_HANDLE;
   }
-  CK_OBJECT_CLASS oClass = CKO_PRIVATE_KEY;
   CK_OBJECT_HANDLE hObject = CK_INVALID_HANDLE;
   CK_ULONG objectCount = 0;
+  CK_ULONG labelLen = 0;
+  if(label) labelLen = strlen(label);
 
-  CK_ATTRIBUTE objTemplate[] = {
+  CK_ATTRIBUTE objTemplateID[] = {
     { CKA_CLASS, &oClass, sizeof(oClass) },
     { CKA_ID,    objID,   objIDLen }
   };
+  CK_ATTRIBUTE objTemplateLabel[] = {
+    { CKA_CLASS, &oClass, sizeof(oClass) },
+    { CKA_LABEL, label,   labelLen }
+  };
 
-  CK_RV rv = p11->C_FindObjectsInit(hSession, objTemplate, 2);
+  CK_RV rv;
+  if(objID != NULL) {
+    rv = p11->C_FindObjectsInit(hSession, objTemplateID, 2);
+  } else {
+    rv = p11->C_FindObjectsInit(hSession, objTemplateLabel, 2);
+  }
   if(rv != CKR_OK) {
     fprintf(stderr, "Error: Could not prepare the object search.\n");
     return CK_INVALID_HANDLE;
@@ -1074,7 +1257,7 @@ CK_OBJECT_HANDLE searchObject(CK_SESSION_HANDLE hSession, char *objID, int objID
 
 // Write the key pair to disk
 
-CK_RV writeKeyToDisk(char *filePath, char *filePIN, Private_Key *privKey) {
+CK_RV writeKeyToDisk(char *filePath, char *filePIN, Botan::Private_Key *privKey) {
   if(filePath == NULL || privKey == NULL) {
     return CKR_GENERAL_ERROR;
   }
@@ -1086,13 +1269,13 @@ CK_RV writeKeyToDisk(char *filePath, char *filePIN, Private_Key *privKey) {
     return CKR_GENERAL_ERROR;
   }
 
-  AutoSeeded_RNG *rng = new AutoSeeded_RNG();
+  Botan::AutoSeeded_RNG *rng = new Botan::AutoSeeded_RNG();
 
   try {
     if(filePIN == NULL) {
-      privFile << PKCS8::PEM_encode(*privKey);
+      privFile << Botan::PKCS8::PEM_encode(*privKey);
     } else {
-      privFile << PKCS8::PEM_encode(*privKey, *rng, filePIN);
+      privFile << Botan::PKCS8::PEM_encode(*privKey, *rng, filePIN);
     }
   }
   catch(std::exception& e) {
@@ -1193,8 +1376,8 @@ char* getDBPath(CK_SLOT_ID slotID) {
 
 // Returns a big int of a given attribute.
 
-BigInt getBigIntAttribute(sqlite3_stmt *select_an_attribute_sql, CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE_TYPE type) {
-  BigInt retVal = BigInt(0);
+Botan::BigInt getBigIntAttribute(sqlite3_stmt *select_an_attribute_sql, CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE_TYPE type) {
+  Botan::BigInt retVal = Botan::BigInt(0);
 
   sqlite3_bind_int(select_an_attribute_sql, 1, objectRef);
   sqlite3_bind_int(select_an_attribute_sql, 2, type);
@@ -1205,7 +1388,7 @@ BigInt getBigIntAttribute(sqlite3_stmt *select_an_attribute_sql, CK_OBJECT_HANDL
     CK_ULONG length = sqlite3_column_int(select_an_attribute_sql, 1);
 
     if(pValue != NULL_PTR) {
-      retVal = BigInt((byte *)pValue, (u32bit)length);
+      retVal = Botan::BigInt((Botan::byte *)pValue, (Botan::u32bit)length);
     }
   }
 
@@ -1262,24 +1445,24 @@ CK_KEY_TYPE getKeyType(sqlite3_stmt *select_an_attribute_sql, CK_OBJECT_HANDLE o
 
 // Get the private key from database
 
-Private_Key* getPrivKey(char *dbPath, CK_OBJECT_HANDLE oHandle) {
+Botan::Private_Key* getPrivKey(char *dbPath, CK_OBJECT_HANDLE oHandle) {
   sqlite3 *db = NULL;
   const char select_str[] = "SELECT value,length FROM Attributes WHERE objectID = ? AND type = ?;";
   sqlite3_stmt *select_sql = NULL;
-  Private_Key *privKey = NULL;
+  Botan::Private_Key *privKey = NULL;
 
   if(sqlite3_open(dbPath, &db) == 0 && sqlite3_prepare_v2(db, select_str, -1, &select_sql, NULL) == 0) {
     if(getObjectClass(select_sql, oHandle) == CKO_PRIVATE_KEY && getKeyType(select_sql, oHandle) == CKK_RSA) {
-      BigInt bigN = getBigIntAttribute(select_sql, oHandle, CKA_MODULUS);
-      BigInt bigE = getBigIntAttribute(select_sql, oHandle, CKA_PUBLIC_EXPONENT);
-      BigInt bigD = getBigIntAttribute(select_sql, oHandle, CKA_PRIVATE_EXPONENT);
-      BigInt bigP = getBigIntAttribute(select_sql, oHandle, CKA_PRIME_1);
-      BigInt bigQ = getBigIntAttribute(select_sql, oHandle, CKA_PRIME_2);
+      Botan::BigInt bigN = getBigIntAttribute(select_sql, oHandle, CKA_MODULUS);
+      Botan::BigInt bigE = getBigIntAttribute(select_sql, oHandle, CKA_PUBLIC_EXPONENT);
+      Botan::BigInt bigD = getBigIntAttribute(select_sql, oHandle, CKA_PRIVATE_EXPONENT);
+      Botan::BigInt bigP = getBigIntAttribute(select_sql, oHandle, CKA_PRIME_1);
+      Botan::BigInt bigQ = getBigIntAttribute(select_sql, oHandle, CKA_PRIME_2);
 
-      AutoSeeded_RNG *rng = new AutoSeeded_RNG();
+      Botan::AutoSeeded_RNG *rng = new Botan::AutoSeeded_RNG();
       
       try {
-        privKey = new RSA_PrivateKey(*rng, bigP, bigQ, bigE, bigD, bigN);
+        privKey = new Botan::RSA_PrivateKey(*rng, bigP, bigQ, bigE, bigD, bigN);
       }
       catch(...) {
         fprintf(stderr, "Error: Could not extract the private key material from database.\n");
