@@ -1,4 +1,4 @@
-/* $Id: softhsm.h 3905 2010-09-09 11:03:16Z rb $ */
+/* $Id: softhsm.h 5299 2011-07-07 11:09:10Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -29,13 +29,12 @@
 #ifndef SOFTHSM_SOFTHSM_H
 #define SOFTHSM_SOFTHSM_H 1
 
-#include "pkcs11.h"
+#include "cryptoki.h"
 #include <sqlite3.h>
 
 // Includes for the crypto library
 #include <botan/pk_keys.h>
 #include <botan/bigint.h>
-using namespace Botan;
 
 typedef struct key_material_t {
   CK_ULONG sizeE;
@@ -82,6 +81,7 @@ void showSlots();
 void importKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, char *objectLabel, char *objectID, int forceExec);
 void exportKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, char *objectID);
 void optimize(char *slot, char *userPIN);
+void trustObject(char *boolTrusted, char *slot, char *soPIN, char *type, char *label, char *objectID);
 
 // Support functions
 
@@ -94,10 +94,10 @@ key_material_t* importKeyMat(char *filePath, char *filePIN);
 void freeKeyMaterial(key_material_t *keyMaterial);
 
 /// DB info
-Private_Key* getPrivKey(char *dbPath, CK_OBJECT_HANDLE oHandle);
+Botan::Private_Key* getPrivKey(char *dbPath, CK_OBJECT_HANDLE oHandle);
 CK_KEY_TYPE getKeyType(sqlite3_stmt *select_an_attribute_sql, CK_OBJECT_HANDLE objectRef);
 CK_OBJECT_CLASS getObjectClass(sqlite3_stmt *select_an_attribute_sql, CK_OBJECT_HANDLE objectRef);
-BigInt getBigIntAttribute(sqlite3_stmt *select_an_attribute_sql, CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE_TYPE type);
+Botan::BigInt getBigIntAttribute(sqlite3_stmt *select_an_attribute_sql, CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE_TYPE type);
 int removeSessionObjs(char *dbPath);
 
 /// Config
@@ -107,9 +107,9 @@ static void *moduleHandle;
 static CK_FUNCTION_LIST_PTR p11;
 
 /// PKCS#11 support
-CK_OBJECT_HANDLE searchObject(CK_SESSION_HANDLE hSession, char *objID, int objIDLen);
+CK_OBJECT_HANDLE searchObject(CK_SESSION_HANDLE hSession, CK_OBJECT_CLASS oClass, char *label, char *objID, int objIDLen);
 
 /// Key to file
-CK_RV writeKeyToDisk(char *filePath, char *filePIN, Private_Key *privKey);
+CK_RV writeKeyToDisk(char *filePath, char *filePIN, Botan::Private_Key *privKey);
 
 #endif /* SOFTHSM_SOFTHSM_H */
