@@ -1,4 +1,4 @@
-/* $Id: SoftDatabase.cpp 4675 2011-04-01 13:45:41Z rb $ */
+/* $Id: SoftDatabase.cpp 5229 2011-06-13 12:05:24Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -274,7 +274,7 @@ CK_RV SoftDatabase::saveTokenInfo(int valueID, char *value, int length) {
 // Makes sure that object is saved with all its attributes.
 // If some error occur when saving the data, nothing is saved.
 
-CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPub(RSA_PrivateKey *rsaKey, CK_ATTRIBUTE_PTR pPublicKeyTemplate, 
+CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPub(CK_STATE state, Botan::RSA_PrivateKey *rsaKey, CK_ATTRIBUTE_PTR pPublicKeyTemplate, 
     CK_ULONG ulPublicKeyAttributeCount) {
 
   // Begin the transaction
@@ -318,12 +318,13 @@ CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPub(RSA_PrivateKey *rsaKey, CK_ATTRIBUTE
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_VERIFY, &ckTrue, sizeof(ckTrue)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_VERIFY_RECOVER, &ckTrue, sizeof(ckTrue)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_WRAP, &ckTrue, sizeof(ckTrue)) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_TRUSTED, &ckFalse, sizeof(ckFalse)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_START_DATE, &emptyDate, 0) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_END_DATE, &emptyDate, 0) != CKR_OK);
 
   // The RSA modulus bits
-  IF_Scheme_PublicKey *ifKey = dynamic_cast<IF_Scheme_PublicKey*>(rsaKey);
-  BigInt bigModulus = ifKey->get_n();
+  Botan::IF_Scheme_PublicKey *ifKey = dynamic_cast<Botan::IF_Scheme_PublicKey*>(rsaKey);
+  Botan::BigInt bigModulus = ifKey->get_n();
   CK_ULONG bits = bigModulus.bits();
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_MODULUS_BITS, &bits, sizeof(bits)) != CKR_OK);
 
@@ -331,9 +332,11 @@ CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPub(RSA_PrivateKey *rsaKey, CK_ATTRIBUTE
   CHECK_DB_RESPONSE(this->saveAttributeBigInt(objectID, CKA_MODULUS, &bigModulus) != CKR_OK);
 
   // The RSA public exponent
-  BigInt bigExponent = ifKey->get_e();
+  Botan::BigInt bigExponent = ifKey->get_e();
   CHECK_DB_RESPONSE(this->saveAttributeBigInt(objectID, CKA_PUBLIC_EXPONENT, &bigExponent) != CKR_OK);
 
+  CK_BBOOL trusted;
+
   // Extract the attributes from the template
   for(CK_ULONG i = 0; i < ulPublicKeyAttributeCount; i++) {
     switch(pPublicKeyTemplate[i].type) {
@@ -353,12 +356,20 @@ CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPub(RSA_PrivateKey *rsaKey, CK_ATTRIBUTE
       case CKA_VERIFY:
       case CKA_VERIFY_RECOVER:
       case CKA_WRAP:
-      case CKA_TRUSTED:
         if(pPublicKeyTemplate[i].ulValueLen == sizeof(CK_BBOOL)) {
           CHECK_DB_RESPONSE(this->saveAttribute(objectID, pPublicKeyTemplate[i].type, pPublicKeyTemplate[i].pValue, 
                             pPublicKeyTemplate[i].ulValueLen) != CKR_OK);
         }
         break;
+      case CKA_TRUSTED:
+        // Check for the correct size
+        CHECK_DB_RESPONSE(pPublicKeyTemplate[i].ulValueLen != sizeof(CK_BBOOL))
+        // CKA_TRUSTED can only be set to true by SO
+        trusted = *(CK_BBOOL*)pPublicKeyTemplate[i].pValue;
+        CHECK_DB_RESPONSE(state != CKS_RW_SO_FUNCTIONS && trusted != CK_FALSE)
+        CHECK_DB_RESPONSE(this->saveAttribute(objectID, pPublicKeyTemplate[i].type, pPublicKeyTemplate[i].pValue, 
+                          pPublicKeyTemplate[i].ulValueLen) != CKR_OK);
+        break;
       // Date
       case CKA_START_DATE:
       case CKA_END_DATE:
@@ -384,7 +395,7 @@ CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPub(RSA_PrivateKey *rsaKey, CK_ATTRIBUTE
 // Makes sure that object is saved with all its attributes.
 // If some error occur when saving the data, nothing is saved.
 
-CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPriv(RSA_PrivateKey *rsaKey, CK_ATTRIBUTE_PTR pPrivateKeyTemplate, 
+CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPriv(CK_STATE state, Botan::RSA_PrivateKey *rsaKey, CK_ATTRIBUTE_PTR pPrivateKeyTemplate, 
     CK_ULONG ulPrivateKeyAttributeCount) {
 
   // Begin the transaction
@@ -425,6 +436,7 @@ CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPriv(RSA_PrivateKey *rsaKey, CK_ATTRIBUT
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_TOKEN, &ckFalse, sizeof(ckFalse)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_DERIVE, &ckFalse, sizeof(ckFalse)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_WRAP_WITH_TRUSTED, &ckTrue, sizeof(ckTrue)) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_ALWAYS_AUTHENTICATE, &ckFalse, sizeof(ckFalse)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_SENSITIVE, &ckTrue, sizeof(ckTrue)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_ALWAYS_SENSITIVE, &ckTrue, sizeof(ckTrue)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_DECRYPT, &ckTrue, sizeof(ckTrue)) != CKR_OK);
@@ -437,24 +449,24 @@ CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPriv(RSA_PrivateKey *rsaKey, CK_ATTRIBUT
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_END_DATE, &emptyDate, 0) != CKR_OK);
 
   // The RSA modulus
-  IF_Scheme_PrivateKey *ifKeyPriv = dynamic_cast<IF_Scheme_PrivateKey*>(rsaKey);
-  BigInt bigMod = ifKeyPriv->get_n();
+  Botan::IF_Scheme_PrivateKey *ifKeyPriv = dynamic_cast<Botan::IF_Scheme_PrivateKey*>(rsaKey);
+  Botan::BigInt bigMod = ifKeyPriv->get_n();
   this->saveAttributeBigInt(objectID, CKA_MODULUS, &bigMod);
 
   // The RSA public exponent
-  BigInt bigExp = ifKeyPriv->get_e();
+  Botan::BigInt bigExp = ifKeyPriv->get_e();
   this->saveAttributeBigInt(objectID, CKA_PUBLIC_EXPONENT, &bigExp);
 
   // The RSA private exponent
-  BigInt bigPrivExp = ifKeyPriv->get_d();
+  Botan::BigInt bigPrivExp = ifKeyPriv->get_d();
   this->saveAttributeBigInt(objectID, CKA_PRIVATE_EXPONENT, &bigPrivExp);
 
   // The RSA prime p
-  BigInt bigPrime1 = ifKeyPriv->get_p();
+  Botan::BigInt bigPrime1 = ifKeyPriv->get_p();
   this->saveAttributeBigInt(objectID, CKA_PRIME_1, &bigPrime1);
 
   // The RSA prime q
-  BigInt bigPrime2 = ifKeyPriv->get_q();
+  Botan::BigInt bigPrime2 = ifKeyPriv->get_q();
   this->saveAttributeBigInt(objectID, CKA_PRIME_2, &bigPrime2);
 
   CK_BBOOL bolVal;
@@ -479,6 +491,7 @@ CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPriv(RSA_PrivateKey *rsaKey, CK_ATTRIBUT
       case CKA_SIGN_RECOVER:
       case CKA_UNWRAP:
       case CKA_WRAP_WITH_TRUSTED:
+      case CKA_ALWAYS_AUTHENTICATE:
         if(pPrivateKeyTemplate[i].ulValueLen == sizeof(CK_BBOOL)) {
           CHECK_DB_RESPONSE(this->saveAttribute(objectID, pPrivateKeyTemplate[i].type, pPrivateKeyTemplate[i].pValue, 
                             pPrivateKeyTemplate[i].ulValueLen) != CKR_OK);
@@ -525,6 +538,63 @@ CK_OBJECT_HANDLE SoftDatabase::addRSAKeyPriv(RSA_PrivateKey *rsaKey, CK_ATTRIBUT
   return objectID;
 }
 
+// Import a certificate. The template has been validated.
+
+CK_OBJECT_HANDLE SoftDatabase::importPublicCert(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount) {
+  // Begin the transaction
+  int retVal = 0;
+  while((retVal = sqlite3_exec(db, "BEGIN IMMEDIATE;", NULL, NULL, NULL)) == SQLITE_BUSY) {
+    sched_yield();
+  }
+  if(retVal != SQLITE_OK) {
+    return CK_INVALID_HANDLE;
+  }
+
+  CHECK_DB_RESPONSE(sqlite3_step(insert_object_sql) != SQLITE_DONE);
+  CK_OBJECT_HANDLE objectID = sqlite3_last_insert_rowid(db);
+  sqlite3_reset(insert_object_sql);
+
+  CK_BBOOL ckTrue = CK_TRUE, ckFalse = CK_FALSE;
+  CK_DATE emptyDate;
+  CK_ULONG zero = 0;
+
+  // Created by db handle. So we can remove the correct session objects in the future.
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_VENDOR_DEFINED, &db, sizeof(db)) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_VENDOR_DEFINED+1, appID, strlen(appID)) != CKR_OK);
+
+  // Default values, may be changed by the template.
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_TOKEN, &ckFalse, sizeof(ckFalse)) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_PRIVATE, &ckTrue, sizeof(ckTrue)) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_MODIFIABLE, &ckTrue, sizeof(ckTrue)) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_LABEL, NULL_PTR, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_TRUSTED, &ckFalse, sizeof(ckFalse)) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_CERTIFICATE_CATEGORY, &zero, sizeof(zero)) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_CHECK_VALUE, NULL_PTR, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_START_DATE, &emptyDate, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_END_DATE, &emptyDate, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_SUBJECT, NULL_PTR, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_ID, NULL_PTR, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_ISSUER, NULL_PTR, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_SERIAL_NUMBER, NULL_PTR, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_VALUE, NULL_PTR, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_URL, NULL_PTR, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_HASH_OF_SUBJECT_PUBLIC_KEY, NULL_PTR, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_HASH_OF_ISSUER_PUBLIC_KEY, NULL_PTR, 0) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_JAVA_MIDP_SECURITY_DOMAIN, &zero, sizeof(zero)) != CKR_OK);
+
+  // Extract the attributes
+  for(CK_ULONG i = 0; i < ulCount; i++) {
+    CHECK_DB_RESPONSE(this->saveAttribute(objectID, pTemplate[i].type, pTemplate[i].pValue,
+                      pTemplate[i].ulValueLen) != CKR_OK);
+  }
+
+  while(sqlite3_exec(db, "COMMIT;", NULL, NULL, NULL) == SQLITE_BUSY) {
+    sched_yield();
+  }
+
+  return objectID;
+}
+
 // Import an external public key. The template has been validated.
 
 CK_OBJECT_HANDLE SoftDatabase::importPublicKey(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount) {
@@ -565,14 +635,15 @@ CK_OBJECT_HANDLE SoftDatabase::importPublicKey(CK_ATTRIBUTE_PTR pTemplate, CK_UL
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_VERIFY, &ckTrue, sizeof(ckTrue)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_VERIFY_RECOVER, &ckTrue, sizeof(ckTrue)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_WRAP, &ckTrue, sizeof(ckTrue)) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_TRUSTED, &ckFalse, sizeof(ckFalse)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_START_DATE, &emptyDate, 0) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_END_DATE, &emptyDate, 0) != CKR_OK);
 
   // Extract the attributes
   for(CK_ULONG i = 0; i < ulCount; i++) {
     if(pTemplate[i].type == CKA_MODULUS) {
-      BigInt bigModulus = BigInt(0);
-      bigModulus.binary_decode((byte*)pTemplate[i].pValue, pTemplate[i].ulValueLen);
+      Botan::BigInt bigModulus = Botan::BigInt(0);
+      bigModulus.binary_decode((Botan::byte*)pTemplate[i].pValue, pTemplate[i].ulValueLen);
       CK_ULONG bits = bigModulus.bits();
       CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_MODULUS_BITS, &bits, sizeof(bits)) != CKR_OK);
     }
@@ -624,6 +695,7 @@ CK_OBJECT_HANDLE SoftDatabase::importPrivateKey(CK_ATTRIBUTE_PTR pTemplate, CK_U
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_TOKEN, &ckFalse, sizeof(ckFalse)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_DERIVE, &ckFalse, sizeof(ckFalse)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_WRAP_WITH_TRUSTED, &ckTrue, sizeof(ckTrue)) != CKR_OK);
+  CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_ALWAYS_AUTHENTICATE, &ckFalse, sizeof(ckFalse)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_SENSITIVE, &ckTrue, sizeof(ckTrue)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_ALWAYS_SENSITIVE, &ckTrue, sizeof(ckTrue)) != CKR_OK);
   CHECK_DB_RESPONSE(this->saveAttribute(objectID, CKA_DECRYPT, &ckTrue, sizeof(ckTrue)) != CKR_OK);
@@ -719,7 +791,7 @@ CK_RV SoftDatabase::saveAttribute(CK_OBJECT_HANDLE objectID, CK_ATTRIBUTE_TYPE t
 
 // Convert the big integer and save it in the database.
 
-CK_RV SoftDatabase::saveAttributeBigInt(CK_OBJECT_HANDLE objectID, CK_ATTRIBUTE_TYPE type, BigInt *bigNumber) {
+CK_RV SoftDatabase::saveAttributeBigInt(CK_OBJECT_HANDLE objectID, CK_ATTRIBUTE_TYPE type, Botan::BigInt *bigNumber) {
   CK_ULONG size = bigNumber->bytes();
   CK_VOID_PTR buf = (CK_VOID_PTR)malloc(size);
 
@@ -727,7 +799,7 @@ CK_RV SoftDatabase::saveAttributeBigInt(CK_OBJECT_HANDLE objectID, CK_ATTRIBUTE_
     return CKR_GENERAL_ERROR;
   }
 
-  bigNumber->binary_encode((byte *)buf);
+  bigNumber->binary_encode((Botan::byte *)buf);
 
   CK_RV rv = this->saveAttribute(objectID, type, buf, size);
   free(buf);
@@ -859,9 +931,9 @@ CK_KEY_TYPE SoftDatabase::getKeyType(CK_OBJECT_HANDLE objectRef) {
 // Returns a big int of a given attribute.
 // We reveal anything, because this is used to create a key within the SoftHSM.
 
-BigInt SoftDatabase::getBigIntAttribute(CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE_TYPE type) {
+Botan::BigInt SoftDatabase::getBigIntAttribute(CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE_TYPE type) {
   int retSQL = 0;
-  BigInt retVal = BigInt(0);
+  Botan::BigInt retVal = Botan::BigInt(0);
 
   sqlite3_bind_int(select_an_attribute_sql, 1, objectRef);
   sqlite3_bind_int(select_an_attribute_sql, 2, type);
@@ -877,7 +949,7 @@ BigInt SoftDatabase::getBigIntAttribute(CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE
     CK_ULONG length = sqlite3_column_int(select_an_attribute_sql, 1);
 
     if(pValue != NULL_PTR) {
-      retVal = BigInt((byte *)pValue, (u32bit)length);
+      retVal = Botan::BigInt((Botan::byte *)pValue, (Botan::u32bit)length);
     }
   }
 
@@ -1085,29 +1157,82 @@ CK_RV SoftDatabase::getAttribute(CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTe
 // Set the value of an attribute for this object.
 // This function also performes a sanity check of the template
 
-CK_RV SoftDatabase::setAttribute(CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTemplate) {
+CK_RV SoftDatabase::setAttribute(CK_STATE state, CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTemplate) {
   // Can we modify the object?
   if(this->getBooleanAttribute(objectRef, CKA_MODIFIABLE, CK_FALSE) == CK_FALSE) {
     return CKR_ATTRIBUTE_READ_ONLY;
   }
 
-  // Evaluate the template
+  CK_OBJECT_CLASS oClass = this->getObjectClass(objectRef);
+  CK_RV rv;
+
+  // Evaluate the template for generic objects
   switch(attTemplate->type) {
     case CKA_CLASS:
     case CKA_TOKEN:
     case CKA_PRIVATE:
     case CKA_MODIFIABLE:
+      // We can not change this attribute
+      return CKR_ATTRIBUTE_READ_ONLY;
+    case CKA_LABEL:
+      // We can change
+      break;
+    default:
+      if(oClass == CKO_PUBLIC_KEY) {
+        rv = this->setAttributePublicKey(state, objectRef, attTemplate);
+      } else if(oClass == CKO_PRIVATE_KEY) {
+        rv = this->setAttributePrivateKey(state, objectRef, attTemplate);
+      } else if(oClass == CKO_CERTIFICATE) {
+        rv = this->setAttributeCertificate(state, objectRef, attTemplate);
+      } else {
+        // Invalid attribute
+        return CKR_ATTRIBUTE_TYPE_INVALID;
+      }
+      if(rv != CKR_OK) {
+        return rv;
+      }
+      break;
+  }
+
+  // Save/update in the database
+  this->saveAttribute(objectRef, attTemplate->type, attTemplate->pValue, attTemplate->ulValueLen);
+
+  return CKR_OK;
+}
+
+// Set the value of an attribute for this object.
+// This function also performes a sanity check of the template
+
+CK_RV SoftDatabase::setAttributePublicKey(CK_STATE state, CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTemplate) {
+  CK_BBOOL trusted;
+
+  // Evaluate the template for public key objects
+  switch(attTemplate->type) {
     case CKA_KEY_TYPE:
     case CKA_LOCAL:
     case CKA_KEY_GEN_MECHANISM:
       // We can not change this attribute
       return CKR_ATTRIBUTE_READ_ONLY;
-    case CKA_LABEL:
+    case CKA_TRUSTED:
+      // Check for the correct size
+      if(attTemplate->ulValueLen != sizeof(CK_BBOOL)) {
+        return CKR_ATTRIBUTE_VALUE_INVALID;
+      }
+      // CKA_TRUSTED can only be set to true by SO
+      trusted = *(CK_BBOOL*)attTemplate->pValue;
+      if(state != CKS_RW_SO_FUNCTIONS && trusted != CK_FALSE) {
+        return CKR_ATTRIBUTE_READ_ONLY;
+      }
+      break;
     case CKA_ID:
     case CKA_SUBJECT:
       // We can change
       break;
     case CKA_DERIVE:
+    case CKA_ENCRYPT:
+    case CKA_VERIFY:
+    case CKA_VERIFY_RECOVER:
+    case CKA_WRAP:
       // We can change, but check size
       if(attTemplate->ulValueLen != sizeof(CK_BBOOL)) {
         return CKR_ATTRIBUTE_VALUE_INVALID;
@@ -1121,55 +1246,60 @@ CK_RV SoftDatabase::setAttribute(CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTe
         break;
       }
       return CKR_ATTRIBUTE_VALUE_INVALID;
-    case CKA_ENCRYPT:
-    case CKA_VERIFY:
-    case CKA_VERIFY_RECOVER:
-    case CKA_WRAP:
-      // We can change this for the public key
-      // but invalid for other object classes
-      if(this->getObjectClass(objectRef) != CKO_PUBLIC_KEY) {
-        return CKR_ATTRIBUTE_TYPE_INVALID;
-      }
-      // Check size
-      if(attTemplate->ulValueLen != sizeof(CK_BBOOL)) {
-        return CKR_ATTRIBUTE_VALUE_INVALID;
-      }
-      break;
-    case CKA_TRUSTED:
-      // We can not set this for the public key
-      if(this->getObjectClass(objectRef) == CKO_PUBLIC_KEY) {
+    case CKA_PUBLIC_EXPONENT:
+    case CKA_MODULUS:
+    case CKA_MODULUS_BITS:
+      // We can not set this for the RSA key
+      if(this->getKeyType(objectRef) == CKK_RSA) {
         return CKR_ATTRIBUTE_READ_ONLY;
       }
-      // Invalid for other object classes
+      // Invalid for other objects
       return CKR_ATTRIBUTE_TYPE_INVALID;
+    default:
+      // Invalid attribute
+      return CKR_ATTRIBUTE_TYPE_INVALID;
+  }
+
+  return CKR_OK;
+}
+
+// Set the value of an attribute for this object.
+// This function also performes a sanity check of the template
+
+CK_RV SoftDatabase::setAttributePrivateKey(CK_STATE state, CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTemplate) {
+  // Evaluate the template for private key objects
+  switch(attTemplate->type) {
+    case CKA_KEY_TYPE:
+    case CKA_LOCAL:
+    case CKA_KEY_GEN_MECHANISM:
+    case CKA_ALWAYS_SENSITIVE:
+    case CKA_NEVER_EXTRACTABLE:
+      // We can not change this attribute
+      return CKR_ATTRIBUTE_READ_ONLY;
+    case CKA_ID:
+    case CKA_SUBJECT:
+      // We can change
+      break;
+    case CKA_DERIVE:
     case CKA_DECRYPT:
     case CKA_SIGN:
     case CKA_SIGN_RECOVER:
     case CKA_UNWRAP:
-      // We can change this for the private key
-      // but invalid for other object classes
-      if(this->getObjectClass(objectRef) != CKO_PRIVATE_KEY) {
-        return CKR_ATTRIBUTE_TYPE_INVALID;
-      }
-      // Check size
+    case CKA_ALWAYS_AUTHENTICATE:
+      // We can change, but check size
       if(attTemplate->ulValueLen != sizeof(CK_BBOOL)) {
         return CKR_ATTRIBUTE_VALUE_INVALID;
       }
       break;
-    case CKA_ALWAYS_SENSITIVE:
-    case CKA_NEVER_EXTRACTABLE:
-      // We can not set this for the private key
-      if(this->getObjectClass(objectRef) == CKO_PRIVATE_KEY) {
-        return CKR_ATTRIBUTE_READ_ONLY;
+    case CKA_START_DATE:
+    case CKA_END_DATE:
+      // We can change, but check size
+      if(attTemplate->ulValueLen == sizeof(CK_DATE) ||
+         attTemplate->ulValueLen == 0) {
+        break;
       }
-      // Invalid for other object classes
-      return CKR_ATTRIBUTE_TYPE_INVALID;
+      return CKR_ATTRIBUTE_VALUE_INVALID;
     case CKA_WRAP_WITH_TRUSTED:
-      // We can change this for the private key
-      // but invalid for other object classes
-      if(this->getObjectClass(objectRef) != CKO_PRIVATE_KEY) {
-        return CKR_ATTRIBUTE_TYPE_INVALID;
-      }
       // Attribute cannot be changed once set to CK_TRUE.
       if(this->getBooleanAttribute(objectRef, CKA_WRAP_WITH_TRUSTED, CK_FALSE) == CK_TRUE) {
         return CKR_ATTRIBUTE_READ_ONLY;
@@ -1180,11 +1310,6 @@ CK_RV SoftDatabase::setAttribute(CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTe
       }
       break;
     case CKA_SENSITIVE:
-      // We can change this for the private key
-      // but invalid for other object classes
-      if(this->getObjectClass(objectRef) != CKO_PRIVATE_KEY) {
-        return CKR_ATTRIBUTE_TYPE_INVALID;
-      }
       // Attribute cannot be changed once set to CK_TRUE.
       if(this->getBooleanAttribute(objectRef, CKA_SENSITIVE, CK_TRUE) == CK_TRUE) {
         return CKR_ATTRIBUTE_READ_ONLY;
@@ -1195,11 +1320,6 @@ CK_RV SoftDatabase::setAttribute(CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTe
       }
       break;
     case CKA_EXTRACTABLE:
-      // We can change this for the private key
-      // but invalid for other object classes
-      if(this->getObjectClass(objectRef) != CKO_PRIVATE_KEY) {
-        return CKR_ATTRIBUTE_TYPE_INVALID;
-      }
       // Attribute cannot be changed once set to CK_FALSE.
       if(this->getBooleanAttribute(objectRef, CKA_EXTRACTABLE, CK_FALSE) == CK_FALSE) {
         return CKR_ATTRIBUTE_READ_ONLY;
@@ -1209,21 +1329,8 @@ CK_RV SoftDatabase::setAttribute(CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTe
         return CKR_ATTRIBUTE_VALUE_INVALID;
       }
       break;
-    case CKA_MODULUS_BITS:
-      // We can not set this for the public rsa key
-      if(this->getObjectClass(objectRef) == CKO_PUBLIC_KEY && this->getKeyType(objectRef) == CKK_RSA) {
-        return CKR_ATTRIBUTE_READ_ONLY;
-      }
-      // Invalid for other object classes
-      return CKR_ATTRIBUTE_TYPE_INVALID;
     case CKA_PUBLIC_EXPONENT:
     case CKA_MODULUS:
-      // We can not set this for the RSA key
-      if(this->getKeyType(objectRef) == CKK_RSA) {
-        return CKR_ATTRIBUTE_READ_ONLY;
-      }
-      // Invalid for other objects
-      return CKR_ATTRIBUTE_TYPE_INVALID;
     case CKA_PRIVATE_EXPONENT:
     case CKA_PRIME_1:
     case CKA_PRIME_2:
@@ -1231,7 +1338,7 @@ CK_RV SoftDatabase::setAttribute(CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTe
     case CKA_EXPONENT_2:
     case CKA_COEFFICIENT:
       // We can not set this for the private RSA key
-      if(this->getObjectClass(objectRef) == CKO_PRIVATE_KEY && this->getKeyType(objectRef) == CKK_RSA) {
+      if(this->getKeyType(objectRef) == CKK_RSA) {
         return CKR_ATTRIBUTE_READ_ONLY;
       }
       // Invalid for other objects
@@ -1241,8 +1348,62 @@ CK_RV SoftDatabase::setAttribute(CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTe
       return CKR_ATTRIBUTE_TYPE_INVALID;
   }
 
-  // Save/update in the database
-  this->saveAttribute(objectRef, attTemplate->type, attTemplate->pValue, attTemplate->ulValueLen);
+  return CKR_OK;
+}
+
+// Set the value of an attribute for this object.
+// This function also performes a sanity check of the template
+
+CK_RV SoftDatabase::setAttributeCertificate(CK_STATE state, CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE *attTemplate) {
+  CK_BBOOL trusted;
+
+  // Trusted certificates cannot be modified
+  if(this->getBooleanAttribute(objectRef, CKA_TRUSTED, CK_FALSE) == CK_TRUE) {
+    return CKR_ATTRIBUTE_READ_ONLY;
+  }
+
+  // Evaluate the template for certificate objects
+  switch(attTemplate->type) {
+    case CKA_CERTIFICATE_TYPE:
+    case CKA_CERTIFICATE_CATEGORY:
+    case CKA_CHECK_VALUE:
+    case CKA_SUBJECT:
+    case CKA_VALUE:
+    case CKA_URL:
+    case CKA_HASH_OF_SUBJECT_PUBLIC_KEY:
+    case CKA_HASH_OF_ISSUER_PUBLIC_KEY:
+    case CKA_JAVA_MIDP_SECURITY_DOMAIN:
+      // We can not change this attribute
+      return CKR_ATTRIBUTE_READ_ONLY;
+    case CKA_TRUSTED:
+      // Check for the correct size
+      if(attTemplate->ulValueLen != sizeof(CK_BBOOL)) {
+        return CKR_ATTRIBUTE_VALUE_INVALID;
+      }
+      // CKA_TRUSTED can only be set to true by SO
+      trusted = *(CK_BBOOL*)attTemplate->pValue;
+      if(state != CKS_RW_SO_FUNCTIONS && trusted != CK_FALSE) {
+        return CKR_ATTRIBUTE_READ_ONLY;
+      }
+      break;
+    case CKA_ID:
+    case CKA_ISSUER:
+    case CKA_SERIAL_NUMBER:
+      // We can change
+      break;
+    case CKA_START_DATE:
+    case CKA_END_DATE:
+      // We can change, but check size
+      if(attTemplate->ulValueLen == sizeof(CK_DATE) ||
+         attTemplate->ulValueLen == 0) {
+        break;
+      }
+      return CKR_ATTRIBUTE_VALUE_INVALID;
+    default:
+      // Invalid attribute
+      return CKR_ATTRIBUTE_TYPE_INVALID;
+  }
 
   return CKR_OK;
 }
+
