@@ -1,4 +1,4 @@
-/* $Id: SoftHSMInternal.cpp 4666 2011-03-29 13:27:44Z rb $ */
+/* $Id: SoftHSMInternal.cpp 5347 2011-08-03 12:22:50Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -47,13 +47,13 @@
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <time.h>
 
 // Includes for the crypto library
 #include <botan/pipe.h>
 #include <botan/filters.h>
 #include <botan/hex.h>
 #include <botan/sha2_32.h>
-using namespace Botan;
 
 SoftHSMInternal::SoftHSMInternal() {
   openSessions = 0;
@@ -250,8 +250,16 @@ CK_RV SoftHSMInternal::login(CK_SESSION_HANDLE hSession, CK_USER_TYPE userType,
                      CKR_SESSION_HANDLE_INVALID);
   CHECK_DEBUG_RETURN(pPin == NULL_PTR, "C_Login", "pPin must not be a NULL_PTR",
                      CKR_ARGUMENTS_BAD);
-  CHECK_DEBUG_RETURN(ulPinLen < MIN_PIN_LEN || ulPinLen > MAX_PIN_LEN, "C_Login", "Incorrent PIN length",
-                     CKR_PIN_INCORRECT);
+
+  if(ulPinLen < MIN_PIN_LEN || ulPinLen > MAX_PIN_LEN) {
+    if(userType == CKU_SO) {
+      session->currentSlot->tokenFlags |= CKF_SO_PIN_COUNT_LOW;
+    } else {
+      session->currentSlot->tokenFlags |= CKF_USER_PIN_COUNT_LOW;
+    }
+    DEBUG_MSG("C_Login", "Incorrent PIN length");
+    return CKR_PIN_INCORRECT;
+  }
 
   int logInType = CKU_USER;
 
@@ -301,6 +309,8 @@ CK_RV SoftHSMInternal::login(CK_SESSION_HANDLE hSession, CK_USER_TYPE userType,
     if(strcmp(tmpPIN, session->currentSlot->hashedSOPIN) != 0) {
       free(tmpPIN);
 
+      session->currentSlot->tokenFlags |= CKF_SO_PIN_COUNT_LOW;
+
       DEBUG_MSG("C_Login", "The SO PIN is incorrect");
       return CKR_PIN_INCORRECT;
     }
@@ -316,6 +326,8 @@ CK_RV SoftHSMInternal::login(CK_SESSION_HANDLE hSession, CK_USER_TYPE userType,
       memcpy(session->currentSlot->soPIN, pPin, ulPinLen);
     }
 
+    session->currentSlot->tokenFlags &= ~CKF_SO_PIN_COUNT_LOW;
+
     DEBUG_MSG("C_Login", "OK");
     return CKR_OK;
   } else {
@@ -323,6 +335,8 @@ CK_RV SoftHSMInternal::login(CK_SESSION_HANDLE hSession, CK_USER_TYPE userType,
     if(strcmp(tmpPIN, session->currentSlot->hashedUserPIN) != 0) {
       free(tmpPIN);
 
+      session->currentSlot->tokenFlags |= CKF_USER_PIN_COUNT_LOW;
+
       DEBUG_MSG("C_Login", "The user PIN is incorrect");
       return CKR_PIN_INCORRECT;
     }
@@ -338,6 +352,8 @@ CK_RV SoftHSMInternal::login(CK_SESSION_HANDLE hSession, CK_USER_TYPE userType,
       memcpy(session->currentSlot->userPIN, pPin, ulPinLen);
     }
 
+    session->currentSlot->tokenFlags &= ~CKF_USER_PIN_COUNT_LOW;
+
     DEBUG_MSG("C_Login", "OK");
     return CKR_OK;
   }
@@ -592,7 +608,7 @@ CK_RV SoftHSMInternal::setAttributeValue(CK_SESSION_HANDLE hSession, CK_OBJECT_H
 
   // Loop through all the attributes in the template
   for(CK_ULONG i = 0; i < ulCount; i++) {
-    objectResult = session->db->setAttribute(hObject, &pTemplate[i]);
+    objectResult = session->db->setAttribute(session->getSessionState(), hObject, &pTemplate[i]);
     if(objectResult != CKR_OK) {
       result = objectResult;
     }
@@ -698,9 +714,14 @@ CK_RV SoftHSMInternal::createObject(CK_SESSION_HANDLE hSession, CK_ATTRIBUTE_PTR
   CK_OBJECT_HANDLE oHandle;
 
   switch(oClass) {
+    case CKO_CERTIFICATE:
+        rv = valAttributeCertificate(session->getSessionState(), pTemplate, ulCount);
+        CHECK_DEBUG_RETURN(rv != CKR_OK, "C_CreateObject", "Problem with object template", rv);
+        oHandle = session->db->importPublicCert(pTemplate, ulCount);
+    	break;
     case CKO_PUBLIC_KEY:
       if(keyType == CKK_RSA) {
-        rv = valAttributePubRSA(pTemplate, ulCount);
+        rv = valAttributePubRSA(session->getSessionState(), pTemplate, ulCount);
         CHECK_DEBUG_RETURN(rv != CKR_OK, "C_CreateObject", "Problem with object template", rv);
         oHandle = session->db->importPublicKey(pTemplate, ulCount);
       } else {
@@ -710,7 +731,7 @@ CK_RV SoftHSMInternal::createObject(CK_SESSION_HANDLE hSession, CK_ATTRIBUTE_PTR
       break;
     case CKO_PRIVATE_KEY:
       if(keyType == CKK_RSA) {
-        rv = valAttributePrivRSA(session->rng, pTemplate, ulCount);
+        rv = valAttributePrivRSA(session->getSessionState(), session->rng, pTemplate, ulCount);
         CHECK_DEBUG_RETURN(rv != CKR_OK, "C_CreateObject", "Problem with object template", rv);
         oHandle = session->db->importPrivateKey(pTemplate, ulCount);
       } else {
@@ -753,6 +774,12 @@ CK_RV SoftHSMInternal::destroyObject(CK_SESSION_HANDLE hSession, CK_OBJECT_HANDL
   for(int i = 0; i < MAX_SESSION_COUNT; i++) {
     if(sessions[i] != NULL_PTR) {
       sessions[i]->keyStore->removeKey(hObject);
+
+      // Also reset the single element cache, so that the old pipe will not be reused
+      // if the handle is reused by a new object.
+      if(sessions[i]->signKey == hObject) {
+        sessions[i]->signKey = CK_INVALID_HANDLE;
+      }
     }
   }
 
