@@ -1,4 +1,4 @@
-/* $Id: SoftHSMInternal.h 4666 2011-03-29 13:27:44Z rb $ */
+/* $Id: SoftHSMInternal.h 5299 2011-07-07 11:09:10Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -36,7 +36,7 @@
 #ifndef SOFTHSM_SOFTHSMINTERNAL_H
 #define SOFTHSM_SOFTHSMINTERNAL_H 1
 
-#include "pkcs11.h"
+#include "cryptoki.h"
 #include "SoftFind.h"
 #include "SoftDatabase.h"
 #include "SoftSession.h"
