@@ -1,4 +1,4 @@
-/* $Id: attribute.cpp 3679 2010-08-09 11:32:32Z rb $ */
+/* $Id: attribute.cpp 5274 2011-06-30 21:32:50Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -33,6 +33,7 @@
 ************************************************************/
 
 #include "attribute.h"
+#include "log.h"
 
 // Standard includes
 #include <stdlib.h>
@@ -41,11 +42,115 @@
 // Includes for the crypto library
 #include <botan/if_algo.h>
 #include <botan/rsa.h>
-using namespace Botan;
 
-CK_RV valAttributePubRSA(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount) {
-  BigInt bigN = BigInt(0);
-  BigInt bigE = BigInt(0);
+CK_RV valAttributeCertificate(CK_STATE state, CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount) {
+  int foundType = 0;
+  int foundSubject = 0;
+  int foundValue = 0;
+  CK_BBOOL trusted;
+
+  // Evaluate the template
+  for(CK_ULONG i = 0; i < ulCount; i++) {
+    if(pTemplate[i].pValue == NULL_PTR && pTemplate[i].ulValueLen > 0) {
+      return CKR_ATTRIBUTE_VALUE_INVALID;
+    }
+    switch(pTemplate[i].type) {
+      case CKA_CLASS:
+        // Check for the correct size
+        if(pTemplate[i].ulValueLen == sizeof(CK_OBJECT_CLASS)) {
+          CK_OBJECT_CLASS oClass = *(CK_OBJECT_CLASS*)pTemplate[i].pValue;
+          if(oClass != CKO_CERTIFICATE) {
+            return CKR_ATTRIBUTE_VALUE_INVALID;
+          }
+        } else {
+          return CKR_ATTRIBUTE_VALUE_INVALID;
+        }
+        break;
+      case CKA_CERTIFICATE_TYPE:
+        // Check for the correct size
+        if(pTemplate[i].ulValueLen == sizeof(CK_CERTIFICATE_TYPE)) {
+          CK_OBJECT_CLASS oType = *(CK_OBJECT_CLASS*)pTemplate[i].pValue;
+          if(oType != CKC_X_509) {
+            return CKR_ATTRIBUTE_VALUE_INVALID;
+          }
+        } else {
+          return CKR_ATTRIBUTE_VALUE_INVALID;
+        }
+        foundType = 1;
+        break;
+      case CKA_TOKEN:
+      case CKA_PRIVATE:
+      case CKA_MODIFIABLE:
+        // Check for the correct size
+        if(pTemplate[i].ulValueLen != sizeof(CK_BBOOL)) {
+          return CKR_ATTRIBUTE_VALUE_INVALID;
+        }
+        break;
+      case CKA_TRUSTED:
+        // Check for the correct size
+        if(pTemplate[i].ulValueLen != sizeof(CK_BBOOL)) {
+          return CKR_ATTRIBUTE_VALUE_INVALID;
+        }
+        // CKA_TRUSTED can only be set to true by SO
+        trusted = *(CK_BBOOL*)pTemplate[i].pValue;
+        if(state != CKS_RW_SO_FUNCTIONS && trusted != CK_FALSE) {
+          return CKR_ATTRIBUTE_READ_ONLY;
+        }
+        break;
+      case CKA_LABEL:
+      case CKA_CHECK_VALUE:
+      case CKA_ID:
+      case CKA_ISSUER:
+      case CKA_SERIAL_NUMBER:
+      case CKA_URL:
+      case CKA_HASH_OF_SUBJECT_PUBLIC_KEY:
+      case CKA_HASH_OF_ISSUER_PUBLIC_KEY:
+        // Variable length, no need to check
+        break;
+      case CKA_SUBJECT:
+        // Variable length, no need to check
+        foundSubject = 1;
+        break;
+      case CKA_VALUE:
+        // Variable length, no need to check
+        foundValue = 1;
+        break;
+      case CKA_CERTIFICATE_CATEGORY:
+      case CKA_JAVA_MIDP_SECURITY_DOMAIN:
+        // Check for the correct size
+        if(pTemplate[i].ulValueLen == sizeof(CK_ULONG)) {
+          CK_ULONG value = *(CK_ULONG*)pTemplate[i].pValue;
+          if(value < 0 || value > 3) {
+            return CKR_ATTRIBUTE_VALUE_INVALID;
+          }
+        } else {
+          return CKR_ATTRIBUTE_VALUE_INVALID;
+        }
+        break;
+      case CKA_START_DATE:
+      case CKA_END_DATE:
+        if(pTemplate[i].ulValueLen != sizeof(CK_DATE) &&
+           pTemplate[i].ulValueLen != 0) {
+          return CKR_ATTRIBUTE_VALUE_INVALID;
+        }
+        break;
+      default:
+        // Invalid attribute
+        return CKR_ATTRIBUTE_TYPE_INVALID;
+    }
+  }
+
+  if(!foundType || !foundSubject || !foundValue) {
+    return CKR_TEMPLATE_INCOMPLETE;
+  } 
+
+  return CKR_OK;
+}
+
+CK_RV valAttributePubRSA(CK_STATE state, CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount) {
+  Botan::BigInt bigN = Botan::BigInt(0);
+  Botan::BigInt bigE = Botan::BigInt(0);
+  CK_BBOOL trusted;
 
   // Evaluate the template
   for(CK_ULONG i = 0; i < ulCount; i++) {
@@ -104,12 +209,21 @@ CK_RV valAttributePubRSA(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount) {
         // Must not be specified when object is created with C_CreateObject
         return CKR_ATTRIBUTE_VALUE_INVALID;
       case CKA_TRUSTED:
-        return CKR_ATTRIBUTE_READ_ONLY;
+        // Check for the correct size
+        if(pTemplate[i].ulValueLen != sizeof(CK_BBOOL)) {
+          return CKR_ATTRIBUTE_VALUE_INVALID;
+        }
+        // CKA_TRUSTED can only be set to true by SO
+        trusted = *(CK_BBOOL*)pTemplate[i].pValue;
+        if(state != CKS_RW_SO_FUNCTIONS && trusted != CK_FALSE) {
+          return CKR_ATTRIBUTE_READ_ONLY;
+        }
+        break;
       case CKA_PUBLIC_EXPONENT:
-        bigE = BigInt((byte *)pTemplate[i].pValue, (u32bit)pTemplate[i].ulValueLen);
+        bigE = Botan::BigInt((Botan::byte *)pTemplate[i].pValue, (Botan::u32bit)pTemplate[i].ulValueLen);
         break;
       case CKA_MODULUS:
-        bigN = BigInt((byte *)pTemplate[i].pValue, (u32bit)pTemplate[i].ulValueLen);
+        bigN = Botan::BigInt((Botan::byte *)pTemplate[i].pValue, (Botan::u32bit)pTemplate[i].ulValueLen);
         break;
       default:
         // Invalid attribute
@@ -121,11 +235,15 @@ CK_RV valAttributePubRSA(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount) {
     return CKR_TEMPLATE_INCOMPLETE;
   }
 
-  Public_Key *tmpKey = NULL_PTR;
+  Botan::Public_Key *tmpKey = NULL_PTR;
   try {
-    tmpKey = new RSA_PublicKey(bigN, bigE);
+    tmpKey = new Botan::RSA_PublicKey(bigN, bigE);
   }
-  catch(...) {
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not re-create the public key: %s", e.what());
+    ERROR_MSG("valAttributePubRSA", errorMsg);
+
     return CKR_ATTRIBUTE_VALUE_INVALID;
   }
 
@@ -136,12 +254,12 @@ CK_RV valAttributePubRSA(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount) {
   return CKR_OK;
 }
 
-CK_RV valAttributePrivRSA(RandomNumberGenerator *rng, CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount) {
-  BigInt bigN = BigInt(0);
-  BigInt bigE = BigInt(0);
-  BigInt bigD = BigInt(0);
-  BigInt bigP = BigInt(0);
-  BigInt bigQ = BigInt(0);
+CK_RV valAttributePrivRSA(CK_STATE state, Botan::RandomNumberGenerator *rng, CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount) {
+  Botan::BigInt bigN = Botan::BigInt(0);
+  Botan::BigInt bigE = Botan::BigInt(0);
+  Botan::BigInt bigD = Botan::BigInt(0);
+  Botan::BigInt bigP = Botan::BigInt(0);
+  Botan::BigInt bigQ = Botan::BigInt(0);
 
   // Evaluate the template
   for(CK_ULONG i = 0; i < ulCount; i++) {
@@ -180,6 +298,7 @@ CK_RV valAttributePrivRSA(RandomNumberGenerator *rng, CK_ATTRIBUTE_PTR pTemplate
       case CKA_SENSITIVE:
       case CKA_EXTRACTABLE:
       case CKA_WRAP_WITH_TRUSTED:
+      case CKA_ALWAYS_AUTHENTICATE:
         // Check for the correct size
         if(pTemplate[i].ulValueLen != sizeof(CK_BBOOL)) {
           return CKR_ATTRIBUTE_VALUE_INVALID;
@@ -204,19 +323,19 @@ CK_RV valAttributePrivRSA(RandomNumberGenerator *rng, CK_ATTRIBUTE_PTR pTemplate
         // Must not be specified when object is created with C_CreateObject
         return CKR_ATTRIBUTE_VALUE_INVALID;
       case CKA_PUBLIC_EXPONENT:
-        bigE = BigInt((byte *)pTemplate[i].pValue, (u32bit)pTemplate[i].ulValueLen);
+        bigE = Botan::BigInt((Botan::byte *)pTemplate[i].pValue, (Botan::u32bit)pTemplate[i].ulValueLen);
         break;
       case CKA_MODULUS:
-        bigN = BigInt((byte *)pTemplate[i].pValue, (u32bit)pTemplate[i].ulValueLen);
+        bigN = Botan::BigInt((Botan::byte *)pTemplate[i].pValue, (Botan::u32bit)pTemplate[i].ulValueLen);
         break;
       case CKA_PRIVATE_EXPONENT:
-        bigD = BigInt((byte *)pTemplate[i].pValue, (u32bit)pTemplate[i].ulValueLen);
+        bigD = Botan::BigInt((Botan::byte *)pTemplate[i].pValue, (Botan::u32bit)pTemplate[i].ulValueLen);
         break;
       case CKA_PRIME_1:
-        bigP = BigInt((byte *)pTemplate[i].pValue, (u32bit)pTemplate[i].ulValueLen);
+        bigP = Botan::BigInt((Botan::byte *)pTemplate[i].pValue, (Botan::u32bit)pTemplate[i].ulValueLen);
         break;
       case CKA_PRIME_2:
-        bigQ = BigInt((byte *)pTemplate[i].pValue, (u32bit)pTemplate[i].ulValueLen);
+        bigQ = Botan::BigInt((Botan::byte *)pTemplate[i].pValue, (Botan::u32bit)pTemplate[i].ulValueLen);
         break;
       case CKA_EXPONENT_1:
       case CKA_EXPONENT_2:
@@ -233,11 +352,15 @@ CK_RV valAttributePrivRSA(RandomNumberGenerator *rng, CK_ATTRIBUTE_PTR pTemplate
     return CKR_TEMPLATE_INCOMPLETE;
   }
 
-  Public_Key *tmpKey = NULL_PTR;
+  Botan::Public_Key *tmpKey = NULL_PTR;
   try {
-    tmpKey = new RSA_PrivateKey(*rng, bigP, bigQ, bigE, bigD, bigN);
+    tmpKey = new Botan::RSA_PrivateKey(*rng, bigP, bigQ, bigE, bigD, bigN);
   }
-  catch(...) {
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not re-create the private key: %s", e.what());
+    ERROR_MSG("valAttributePrivRSA", errorMsg);
+
     return CKR_ATTRIBUTE_VALUE_INVALID;
   }
 
