@@ -1,4 +1,4 @@
-/* $Id: main.cpp 4673 2011-03-31 11:18:12Z rb $ */
+/* $Id: main.cpp 5277 2011-07-01 12:53:59Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -38,6 +38,7 @@
 #include <config.h>
 #include "main.h"
 #include "log.h"
+#include "botan_compat.h"
 #include "file.h"
 #include "SoftHSMInternal.h"
 #include "userhandling.h"
@@ -65,10 +66,10 @@
 #include <botan/pipe.h>
 #include <botan/emsa3.h>
 #include <botan/emsa_raw.h>
+#include <botan/eme_pkcs.h>
 #include <botan/pk_keys.h>
 #include <botan/bigint.h>
 #include <botan/rsa.h>
-using namespace Botan;
 
 // Keeps the internal state
 std::auto_ptr<SoftHSMInternal> state(NULL);
@@ -212,7 +213,7 @@ CK_RV C_Initialize(CK_VOID_PTR pInitArgs) {
   }
 
   // Init the Botan crypto library 
-  LibraryInitializer::initialize("thread_safe=true");
+  Botan::LibraryInitializer::initialize("thread_safe=true");
 
   DEBUG_MSG("C_Initialize", "OK");
   return CKR_OK;
@@ -233,7 +234,7 @@ CK_RV C_Finalize(CK_VOID_PTR pReserved) {
   state.reset(NULL);
 
   // Deinitialize the Botan crypto lib
-  LibraryInitializer::deinitialize();
+  Botan::LibraryInitializer::deinitialize();
 
   DEBUG_MSG("C_Finalize", "OK");
   return CKR_OK;
@@ -777,18 +778,221 @@ CK_RV C_FindObjectsFinal(CK_SESSION_HANDLE hSession) {
   return CKR_OK;
 }
 
-CK_RV C_EncryptInit(CK_SESSION_HANDLE, CK_MECHANISM_PTR, CK_OBJECT_HANDLE) {
+CK_RV C_EncryptInit(CK_SESSION_HANDLE hSession, CK_MECHANISM_PTR pMechanism, CK_OBJECT_HANDLE hKey) {
   DEBUG_MSG("C_EncryptInit", "Calling");
-  DEBUG_MSG("C_EncryptInit", "The function is not implemented.");
 
-  return CKR_FUNCTION_NOT_SUPPORTED;
+  SoftHSMInternal *softHSM = state.get();
+  CHECK_DEBUG_RETURN(softHSM == NULL, "C_EncryptInit", "Library is not initialized",
+                     CKR_CRYPTOKI_NOT_INITIALIZED);
+  CHECK_DEBUG_RETURN(pMechanism == NULL_PTR, "C_EncryptInit", "pMechanism must not be NULL_PTR",
+                     CKR_ARGUMENTS_BAD);
+
+  SoftSession *session = softHSM->getSession(hSession);
+  if(session == NULL_PTR) {
+    DEBUG_MSG("C_EncryptInit", "Cannot find the session");
+    return CKR_SESSION_HANDLE_INVALID;
+  }
+
+  // Do we have an active operation?
+  if(session->encryptInitialized) {
+    DEBUG_MSG("C_EncryptInit", "Encrypt is already initialized");
+    return CKR_OPERATION_ACTIVE;
+  }
+
+  // Get the key from the session key store.
+  Botan::Public_Key *cryptoKey = session->getKey(hKey);
+  if(cryptoKey == NULL_PTR) {
+    DEBUG_MSG("C_EncryptInit", "The key could not be found");
+    return CKR_KEY_HANDLE_INVALID;
+  }
+
+  // Check if we can access it
+  CK_BBOOL userAuth = userAuthorization(session->getSessionState(), session->db->getBooleanAttribute(hKey, CKA_TOKEN, CK_TRUE),
+                                        session->db->getBooleanAttribute(hKey, CKA_PRIVATE, CK_TRUE), 0);
+  if(userAuth == CK_FALSE) {
+    DEBUG_MSG("C_EncryptInit", "User is not authorized");
+    return CKR_KEY_HANDLE_INVALID;
+  }
+
+  // Is it an RSA public key?
+  if(session->db->getObjectClass(hKey) != CKO_PUBLIC_KEY || session->db->getKeyType(hKey) != CKK_RSA) {
+    DEBUG_MSG("C_EncryptInit", "Only an RSA public key can be used");
+    return CKR_KEY_TYPE_INCONSISTENT;
+  }
+
+  // Does it support encryption?
+  if(session->db->getBooleanAttribute(hKey, CKA_ENCRYPT, CK_TRUE) == CK_FALSE) {
+    DEBUG_MSG("C_EncryptInit", "This key does not support encryption");
+    return CKR_KEY_FUNCTION_NOT_PERMITTED;
+  }
+
+  session->encryptSinglePart = false;
+
+#ifdef BOTAN_PRE_1_9_4_FIX
+  Botan::EME *eme = NULL_PTR;
+
+  // Selects the correct padding.
+  switch(pMechanism->mechanism) {
+    case CKM_RSA_PKCS:
+      eme = new Botan::EME_PKCS1v15();
+      session->encryptSinglePart = true;
+      break;
+    default:
+      DEBUG_MSG("C_EncryptInit", "The selected mechanism is not supported");
+      return CKR_MECHANISM_INVALID;
+      break;
+  }
+
+  if(eme == NULL_PTR) {
+    DEBUG_MSG("C_EncryptInit", "Could not create the padding");
+    return CKR_DEVICE_MEMORY;
+  }
+#else
+  std::string eme;
+
+  // Selects the correct padding.
+  switch(pMechanism->mechanism) {
+    case CKM_RSA_PKCS:
+      eme = "EME-PKCS1-v1_5";
+      session->encryptSinglePart = true;
+      break;
+    default:
+      DEBUG_MSG("C_EncryptInit", "The selected mechanism is not supported");
+      return CKR_MECHANISM_INVALID;
+      break;
+  }
+#endif
+
+  // Creates the encryptor with given key and mechanism.
+  try {
+#ifdef BOTAN_PRE_1_9_4_FIX
+    Botan::PK_Encrypting_Key *encryptKey = dynamic_cast<Botan::PK_Encrypting_Key*>(cryptoKey);
+    session->encryptSize = (cryptoKey->max_input_bits() + 8) / 8;
+    session->pkEncryptor = new Botan::PK_Encryptor_MR_with_EME(*encryptKey, &*eme);
+#else
+    session->encryptSize = (cryptoKey->max_input_bits() + 8) / 8;
+    session->pkEncryptor = new Botan::PK_Encryptor_EME(*cryptoKey, eme);
+#endif
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not create the encryption function: %s", e.what());
+    ERROR_MSG("C_EncryptInit", errorMsg);
+    return CKR_GENERAL_ERROR;
+  }
+
+  if(!session->pkEncryptor) {
+    ERROR_MSG("C_EncryptInit", "Could not create the encryption function");
+    return CKR_DEVICE_MEMORY;
+  }
+
+  session->encryptInitialized = true;
+
+  DEBUG_MSG("C_EncryptInit", "OK");
+  return CKR_OK;
 }
 
-CK_RV C_Encrypt(CK_SESSION_HANDLE, CK_BYTE_PTR, CK_ULONG, CK_BYTE_PTR, CK_ULONG_PTR) {
+CK_RV C_Encrypt(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pData, CK_ULONG ulDataLen, 
+                CK_BYTE_PTR pEncryptedData, CK_ULONG_PTR pulEncryptedDataLen) {
   DEBUG_MSG("C_Encrypt", "Calling");
-  DEBUG_MSG("C_Encrypt", "The function is not implemented.");
 
-  return CKR_FUNCTION_NOT_SUPPORTED;
+  SoftHSMInternal *softHSM = state.get();
+  CHECK_DEBUG_RETURN(softHSM == NULL, "C_Encrypt", "Library is not initialized",
+                     CKR_CRYPTOKI_NOT_INITIALIZED);
+
+  SoftSession *session = softHSM->getSession(hSession);
+  if(session == NULL_PTR) {
+    DEBUG_MSG("C_Encrypt", "Cannot find the session");
+    return CKR_SESSION_HANDLE_INVALID;
+  }
+
+  if(!session->encryptInitialized) {
+    DEBUG_MSG("C_Encrypt", "Encrypt is not initialized");
+    return CKR_OPERATION_NOT_INITIALIZED;
+  }
+
+  if(pulEncryptedDataLen == NULL_PTR) {
+    DEBUG_MSG("C_Encrypt", "pulEncryptedDataLen must not be a NULL_PTR")
+
+    // Finalizing
+    session->encryptSize = 0;
+    delete session->pkEncryptor;
+    session->pkEncryptor = NULL_PTR;
+    session->encryptInitialized = false;
+
+    return CKR_ARGUMENTS_BAD;
+  }
+
+  if(pEncryptedData == NULL_PTR) {
+    *pulEncryptedDataLen = session->encryptSize;
+
+    DEBUG_MSG("C_Encrypt", "OK, returning the size of the encrypted data");
+    return CKR_OK;
+  }
+
+  if(*pulEncryptedDataLen < session->encryptSize) {
+    *pulEncryptedDataLen = session->encryptSize;
+
+    DEBUG_MSG("C_Encrypt", "The given buffer is too small");
+    return CKR_BUFFER_TOO_SMALL;
+  }
+
+  if(pData == NULL_PTR) {
+    DEBUG_MSG("C_Encrypt", "pData must not be a NULL_PTR");
+
+    // Finalizing
+    session->encryptSize = 0;
+    delete session->pkEncryptor;
+    session->pkEncryptor = NULL_PTR;
+    session->encryptInitialized = false;
+
+    return CKR_ARGUMENTS_BAD;
+  }
+
+  // Check input size
+  if(session->pkEncryptor->maximum_input_size() < ulDataLen) {
+    ERROR_MSG("C_Encrypt", "Input data is too large");
+
+    // Finalizing
+    session->encryptSize = 0;
+    delete session->pkEncryptor;
+    session->pkEncryptor = NULL_PTR;
+    session->encryptInitialized = false;
+
+    return CKR_DATA_LEN_RANGE;
+  }
+
+  // Encrypt
+  Botan::SecureVector<Botan::byte> encryptResult;
+  try {
+    encryptResult = session->pkEncryptor->encrypt(pData, ulDataLen, *session->rng);
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not encrypt the data: %s", e.what());
+    ERROR_MSG("C_Encrypt", errorMsg);
+
+    // Finalizing
+    session->encryptSize = 0;
+    delete session->pkEncryptor;
+    session->pkEncryptor = NULL_PTR;
+    session->encryptInitialized = false;
+
+    return CKR_GENERAL_ERROR;
+  }
+
+  // Returns the result
+  memcpy(pEncryptedData, encryptResult.begin(), encryptResult.size());
+  *pulEncryptedDataLen = encryptResult.size();
+
+  // Finalizing
+  session->encryptSize = 0;
+  delete session->pkEncryptor;
+  session->pkEncryptor = NULL_PTR;
+  session->encryptInitialized = false;
+
+  DEBUG_MSG("C_Encrypt", "OK");
+  return CKR_OK;
 }
 
 CK_RV C_EncryptUpdate(CK_SESSION_HANDLE, CK_BYTE_PTR, CK_ULONG, CK_BYTE_PTR, CK_ULONG_PTR) {
@@ -805,18 +1009,212 @@ CK_RV C_EncryptFinal(CK_SESSION_HANDLE, CK_BYTE_PTR, CK_ULONG_PTR) {
   return CKR_FUNCTION_NOT_SUPPORTED;
 }
 
-CK_RV C_DecryptInit(CK_SESSION_HANDLE, CK_MECHANISM_PTR, CK_OBJECT_HANDLE) {
+CK_RV C_DecryptInit(CK_SESSION_HANDLE hSession, CK_MECHANISM_PTR pMechanism, CK_OBJECT_HANDLE hKey) {
   DEBUG_MSG("C_DecryptInit", "Calling");
-  DEBUG_MSG("C_DecryptInit", "The function is not implemented.");
 
-  return CKR_FUNCTION_NOT_SUPPORTED;
+  SoftHSMInternal *softHSM = state.get();
+  CHECK_DEBUG_RETURN(softHSM == NULL, "C_DecryptInit", "Library is not initialized",
+                     CKR_CRYPTOKI_NOT_INITIALIZED);
+  CHECK_DEBUG_RETURN(pMechanism == NULL_PTR, "C_DecryptInit", "pMechanism must not be NULL_PTR",
+                     CKR_ARGUMENTS_BAD);
+
+  SoftSession *session = softHSM->getSession(hSession);
+  if(session == NULL_PTR) {
+    DEBUG_MSG("C_DecryptInit", "Cannot find the session");
+    return CKR_SESSION_HANDLE_INVALID;
+  }
+
+  // Do we have an active operation?
+  if(session->decryptInitialized) {
+    DEBUG_MSG("C_DecryptInit", "Decrypt is already initialized");
+    return CKR_OPERATION_ACTIVE;
+  }
+
+  // Get the key from the session key store.
+  Botan::Public_Key *cryptoKey = session->getKey(hKey);
+  if(cryptoKey == NULL_PTR) {
+    DEBUG_MSG("C_DecryptInit", "The key could not be found");
+    return CKR_KEY_HANDLE_INVALID;
+  }
+
+  // Check if we can access it
+  CK_BBOOL userAuth = userAuthorization(session->getSessionState(), session->db->getBooleanAttribute(hKey, CKA_TOKEN, CK_TRUE),
+                                        session->db->getBooleanAttribute(hKey, CKA_PRIVATE, CK_TRUE), 0);
+  if(userAuth == CK_FALSE) {
+    DEBUG_MSG("C_DecryptInit", "User is not authorized");
+    return CKR_KEY_HANDLE_INVALID;
+  }
+
+  // Is it an RSA private key?
+  if(session->db->getObjectClass(hKey) != CKO_PRIVATE_KEY || session->db->getKeyType(hKey) != CKK_RSA) {
+    DEBUG_MSG("C_DecryptInit", "Only an RSA private key can be used");
+    return CKR_KEY_TYPE_INCONSISTENT;
+  }
+
+  // Does it support decryption?
+  if(session->db->getBooleanAttribute(hKey, CKA_DECRYPT, CK_TRUE) == CK_FALSE) {
+    DEBUG_MSG("C_DecryptInit", "This key does not support decryption");
+    return CKR_KEY_FUNCTION_NOT_PERMITTED;
+  }
+
+  session->decryptSinglePart = false;
+
+#ifdef BOTAN_PRE_1_9_4_FIX
+  Botan::EME *eme = NULL_PTR;
+
+  // Selects the correct padding.
+  switch(pMechanism->mechanism) {
+    case CKM_RSA_PKCS:
+      eme = new Botan::EME_PKCS1v15();
+      session->decryptSinglePart = true;
+      break;
+    default:
+      DEBUG_MSG("C_DecryptInit", "The selected mechanism is not supported");
+      return CKR_MECHANISM_INVALID;
+      break;
+  }
+
+  if(eme == NULL_PTR) {
+    DEBUG_MSG("C_DecryptInit", "Could not create the padding");
+    return CKR_DEVICE_MEMORY;
+  } 
+#else
+  std::string eme;
+
+  // Selects the correct padding.
+  switch(pMechanism->mechanism) {
+    case CKM_RSA_PKCS:
+      eme = "EME-PKCS1-v1_5";
+      session->decryptSinglePart = true;
+      break;
+    default:
+      DEBUG_MSG("C_DecryptInit", "The selected mechanism is not supported");
+      return CKR_MECHANISM_INVALID;
+      break;
+  }
+#endif
+
+  // Creates the decryptor with given key and mechanism
+  try {
+#ifdef BOTAN_PRE_1_9_4_FIX
+    Botan::PK_Decrypting_Key *decryptKey = dynamic_cast<Botan::PK_Decrypting_Key*>(cryptoKey);
+    session->decryptSize = (cryptoKey->max_input_bits() + 8) / 8;
+    session->pkDecryptor = new Botan::PK_Decryptor_MR_with_EME(*decryptKey, &*eme);
+#else
+    session->decryptSize = (cryptoKey->max_input_bits() + 8) / 8;
+    session->pkDecryptor = new Botan::PK_Decryptor_EME(*dynamic_cast<Botan::Private_Key*>(cryptoKey), eme);
+#endif
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not create the decryption function: %s", e.what());
+    ERROR_MSG("C_DecryptInit", errorMsg);
+    return CKR_GENERAL_ERROR;
+  }
+
+  if(!session->pkDecryptor) {
+    ERROR_MSG("C_DecryptInit", "Could not create the decryption function");
+    return CKR_DEVICE_MEMORY;
+  }
+
+  session->decryptInitialized = true;
+
+  DEBUG_MSG("C_DecryptInit", "OK");
+  return CKR_OK;
 }
 
-CK_RV C_Decrypt(CK_SESSION_HANDLE, CK_BYTE_PTR, CK_ULONG, CK_BYTE_PTR, CK_ULONG_PTR) {
+CK_RV C_Decrypt(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pEncryptedData, CK_ULONG ulEncryptedDataLen, 
+                CK_BYTE_PTR pData, CK_ULONG_PTR pulDataLen) {
   DEBUG_MSG("C_Decrypt", "Calling");
-  DEBUG_MSG("C_Decrypt", "The function is not implemented.");
 
-  return CKR_FUNCTION_NOT_SUPPORTED;
+  SoftHSMInternal *softHSM = state.get();
+  CHECK_DEBUG_RETURN(softHSM == NULL, "C_Decrypt", "Library is not initialized",
+                     CKR_CRYPTOKI_NOT_INITIALIZED);
+
+  SoftSession *session = softHSM->getSession(hSession);
+  if(session == NULL_PTR) {
+    DEBUG_MSG("C_Decrypt", "Cannot find the session");
+    return CKR_SESSION_HANDLE_INVALID;
+  }
+
+  if(!session->decryptInitialized) {
+    DEBUG_MSG("C_Decrypt", "Decrypt is not initialized");
+    return CKR_OPERATION_NOT_INITIALIZED;
+  }
+
+  if(pulDataLen == NULL_PTR) {
+    DEBUG_MSG("C_Decrypt", "pulDataLen must not be a NULL_PTR");
+
+    // Finalizing
+    session->decryptSize = 0;
+    delete session->pkDecryptor;
+    session->pkDecryptor = NULL_PTR;
+    session->decryptInitialized = false;
+
+    return CKR_ARGUMENTS_BAD;
+  }
+
+  // PKCS#11: "This number may somewhat exceed the precise number of 
+  // bytes needed, but should not exceed it by a large amount."
+  //
+  // We return the maximum output that the RSA key can decrypt.
+  // When the data is decrypted, then we know the size.
+
+  if(pData == NULL_PTR) {
+    *pulDataLen = session->decryptSize;
+    DEBUG_MSG("C_Decrypt", "OK, returning the size of the decrypted data");
+    return CKR_OK;
+  }
+
+  if(*pulDataLen < session->decryptSize) {
+    *pulDataLen = session->decryptSize;
+    DEBUG_MSG("C_Decrypt", "The given buffer is too small");
+    return CKR_BUFFER_TOO_SMALL;
+  }
+
+  if(pEncryptedData == NULL_PTR) {
+    DEBUG_MSG("C_Decrypt", "pEncryptedData must not be a NULL_PTR");
+
+    // Finalizing
+    session->decryptSize = 0;
+    delete session->pkDecryptor;
+    session->pkDecryptor = NULL_PTR;
+    session->decryptInitialized = false;
+
+    return CKR_ARGUMENTS_BAD;
+  }
+
+  // Decrypt
+  Botan::SecureVector<Botan::byte> decryptResult;
+  try {
+    decryptResult = session->pkDecryptor->decrypt(pEncryptedData, ulEncryptedDataLen);
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not decrypt the data: %s", e.what());
+    ERROR_MSG("C_Decrypt", errorMsg);
+
+    // Finalizing
+    session->decryptSize = 0;
+    delete session->pkDecryptor;
+    session->pkDecryptor = NULL_PTR;
+    session->decryptInitialized = false;
+
+    return CKR_ENCRYPTED_DATA_INVALID;
+  }
+
+  // Returns the result
+  memcpy(pData, decryptResult.begin(), decryptResult.size());
+  *pulDataLen = decryptResult.size();
+
+  // Finalizing
+  session->decryptSize = 0;
+  delete session->pkDecryptor;
+  session->pkDecryptor = NULL_PTR;
+  session->decryptInitialized = false;
+
+  DEBUG_MSG("C_Decrypt", "OK");
+  return CKR_OK;
 }
 
 CK_RV C_DecryptUpdate(CK_SESSION_HANDLE, CK_BYTE_PTR, CK_ULONG, CK_BYTE_PTR, CK_ULONG_PTR) {
@@ -860,33 +1258,33 @@ CK_RV C_DigestInit(CK_SESSION_HANDLE hSession, CK_MECHANISM_PTR pMechanism) {
   }
 
   CK_ULONG mechSize = 0;
-  HashFunction *hashFunc = NULL_PTR;
+  Botan::HashFunction *hashFunc = NULL_PTR;
 
   // Selects the correct hash algorithm.
   switch(pMechanism->mechanism) {
     case CKM_MD5:
       mechSize = 16;
-      hashFunc = new MD5;
+      hashFunc = new Botan::MD5;
       break;
     case CKM_RIPEMD160:
       mechSize = 20;
-      hashFunc = new RIPEMD_160;
+      hashFunc = new Botan::RIPEMD_160;
       break;
     case CKM_SHA_1:
       mechSize = 20;
-      hashFunc = new SHA_160;
+      hashFunc = new Botan::SHA_160;
       break;
     case CKM_SHA256:
       mechSize = 32;
-      hashFunc = new SHA_256;
+      hashFunc = new Botan::SHA_256;
       break;
     case CKM_SHA384:
       mechSize = 48;
-      hashFunc = new SHA_384;
+      hashFunc = new Botan::SHA_384;
       break;
     case CKM_SHA512:
       mechSize = 64;
-      hashFunc = new SHA_512;
+      hashFunc = new Botan::SHA_512;
       break;
     default:
       DEBUG_MSG("C_DigestInit", "The selected mechanism is not supported");
@@ -901,10 +1299,18 @@ CK_RV C_DigestInit(CK_SESSION_HANDLE hSession, CK_MECHANISM_PTR pMechanism) {
 
   // Creates the digester with given hash algorithm.
   session->digestSize = mechSize;
-  session->digestPipe = new Pipe(new Hash_Filter(hashFunc));
+  try {
+    session->digestPipe = new Botan::Pipe(new Botan::Hash_Filter(hashFunc));
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not create the digesting function: %s", e.what());
+    ERROR_MSG("C_DigestInit", errorMsg);
+    return CKR_GENERAL_ERROR;
+  }
 
   if(!session->digestPipe) {
-    DEBUG_MSG("C_DigestInit", "Could not create the digesting function");
+    ERROR_MSG("C_DigestInit", "Could not create the digesting function");
     return CKR_DEVICE_MEMORY;
   }
 
@@ -959,13 +1365,28 @@ CK_RV C_Digest(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pData, CK_ULONG ulDataLen
     return CKR_ARGUMENTS_BAD;
   }
 
-  // Digest
-  session->digestPipe->write(pData, ulDataLen);
-  session->digestPipe->end_msg();
+  try {
+    // Digest
+    session->digestPipe->write(pData, ulDataLen);
+    session->digestPipe->end_msg();
 
-  // Returns the result
-  session->digestPipe->read(pDigest, session->digestSize);
-  *pulDigestLen = session->digestSize;
+    // Returns the result
+    session->digestPipe->read(pDigest, session->digestSize);
+    *pulDigestLen = session->digestSize;
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not digest the data: %s", e.what());
+    ERROR_MSG("C_Digest", errorMsg);
+
+    // Finalizing
+    session->digestSize = 0;
+    delete session->digestPipe;
+    session->digestPipe = NULL_PTR;
+    session->digestInitialized = false;
+
+    return CKR_GENERAL_ERROR;
+  }
 
   // Finalizing
   session->digestSize = 0;
@@ -1004,7 +1425,22 @@ CK_RV C_DigestUpdate(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pPart, CK_ULONG ulP
   }
 
   // Digest
-  session->digestPipe->write(pPart, ulPartLen);
+  try {
+    session->digestPipe->write(pPart, ulPartLen);
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not digest the data: %s", e.what());
+    ERROR_MSG("C_DigestUpdate", errorMsg);
+
+    // Finalizing
+    session->digestSize = 0;
+    delete session->digestPipe;
+    session->digestPipe = NULL_PTR;
+    session->digestInitialized = false;
+
+    return CKR_GENERAL_ERROR;
+  }
 
   DEBUG_MSG("C_DigestUpdate", "OK");
   return CKR_OK;
@@ -1055,11 +1491,26 @@ CK_RV C_DigestFinal(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pDigest, CK_ULONG_PT
     return CKR_BUFFER_TOO_SMALL;
   }
 
-  session->digestPipe->end_msg();
+  try {
+    session->digestPipe->end_msg();
 
-  // Returns the result
-  session->digestPipe->read(pDigest, session->digestSize);
-  *pulDigestLen = session->digestSize;
+    // Returns the result
+    session->digestPipe->read(pDigest, session->digestSize);
+    *pulDigestLen = session->digestSize;
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not digest the data: %s", e.what());
+    ERROR_MSG("C_DigestFinal", errorMsg);
+
+    // Finalizing
+    session->digestSize = 0;
+    delete session->digestPipe;
+    session->digestPipe = NULL_PTR;
+    session->digestInitialized = false;
+
+    return CKR_GENERAL_ERROR;
+  }
 
   // Finalizing
   session->digestSize = 0;
@@ -1088,7 +1539,7 @@ CK_RV C_SignInit(CK_SESSION_HANDLE hSession, CK_MECHANISM_PTR pMechanism, CK_OBJ
   }
 
   // Get the key from the session key store.
-  Public_Key *cryptoKey = session->getKey(hKey);
+  Botan::Public_Key *cryptoKey = session->getKey(hKey);
 
   // TODO:
   //   Should also add: session->db->getBooleanAttribute(hKey, CKA_SIGN, CK_TRUE) == CK_FALSE
@@ -1118,100 +1569,125 @@ CK_RV C_SignInit(CK_SESSION_HANDLE hSession, CK_MECHANISM_PTR pMechanism, CK_OBJ
     return CKR_ARGUMENTS_BAD;
   }
 
-  session->signSinglePart = false;
-#ifdef BOTAN_PRE_1_9_4_FIX
-  EMSA *hashFunc = NULL_PTR;
-
-  // Selects the correct padding and hash algorithm.
-  switch(pMechanism->mechanism) {
-    case CKM_RSA_PKCS:
-      hashFunc = new EMSA3_Raw();
-      session->signSinglePart = true;
-      break;
-    case CKM_RSA_X_509:
-      hashFunc = new EMSA_Raw();
-      session->signSinglePart = true;
-      break;
-    case CKM_MD5_RSA_PKCS:
-      hashFunc = new EMSA3(new MD5);
-      break;
-    case CKM_RIPEMD160_RSA_PKCS:
-      hashFunc = new EMSA3(new RIPEMD_160);
-      break;
-    case CKM_SHA1_RSA_PKCS:
-      hashFunc = new EMSA3(new SHA_160);
-      break;
-    case CKM_SHA256_RSA_PKCS:
-      hashFunc = new EMSA3(new SHA_256);
-      break;
-    case CKM_SHA384_RSA_PKCS:
-      hashFunc = new EMSA3(new SHA_384);
-      break;
-    case CKM_SHA512_RSA_PKCS:
-      hashFunc = new EMSA3(new SHA_512);
-      break;
-    default:
-      DEBUG_MSG("C_SignInit", "The selected mechanism is not supported");
-      return CKR_MECHANISM_INVALID;
-      break;
-  }
-
-  if(hashFunc == NULL_PTR) {
-    DEBUG_MSG("C_SignInit", "Could not create the hash function");
-    return CKR_DEVICE_MEMORY;
-  }
-#else
-  std::string emsa;
-
-  // Selects the correct padding and hash algorithm.
-  switch(pMechanism->mechanism) {
-    case CKM_RSA_PKCS:
-      emsa = "EMSA3(Raw)";
-      session->signSinglePart = true;
-      break;
-    case CKM_RSA_X_509:
-      emsa = "Raw";
-      session->signSinglePart = true;
-      break;
-    case CKM_MD5_RSA_PKCS:
-      emsa = "EMSA3(MD5)";
-      break;
-    case CKM_RIPEMD160_RSA_PKCS:
-      emsa = "EMSA3(RIPEMD-160)";
-      break;
-    case CKM_SHA1_RSA_PKCS:
-      emsa = "EMSA3(SHA-160)";
-      break;
-    case CKM_SHA256_RSA_PKCS:
-      emsa = "EMSA3(SHA-256)";
-      break;
-    case CKM_SHA384_RSA_PKCS:
-      emsa = "EMSA3(SHA-384)";
-      break;
-    case CKM_SHA512_RSA_PKCS:
-      emsa = "EMSA3(SHA-512)";
-      break;
-    default:
-      DEBUG_MSG("C_SignInit", "The selected mechanism is not supported");
-      return CKR_MECHANISM_INVALID;
-      break;
-  }
+#ifndef BOTAN_NO_PK_SIGNER_REUSE
+  // Check if we cannot reuse the old PK_Signer
+  if(!session->pkSigner || session->signMech != pMechanism->mechanism || session->signKey != hKey) {
 #endif
 
-  // Creates the signer with given key and mechanism.
+    if(session->pkSigner) {
+      delete session->pkSigner;
+      session->pkSigner = NULL;
+    }
+
+    session->signSinglePart = false;
 #ifdef BOTAN_PRE_1_9_4_FIX
-  PK_Signing_Key *signKey = dynamic_cast<PK_Signing_Key*>(cryptoKey);
-  session->signSize = (cryptoKey->max_input_bits() + 8) / 8;
-  session->pkSigner = new PK_Signer(*signKey, &*hashFunc);
+    Botan::EMSA *hashFunc = NULL_PTR;
+
+    // Selects the correct padding and hash algorithm.
+    switch(pMechanism->mechanism) {
+      case CKM_RSA_PKCS:
+        hashFunc = new Botan::EMSA3_Raw();
+        session->signSinglePart = true;
+        break;
+      case CKM_RSA_X_509:
+        hashFunc = new Botan::EMSA_Raw();
+        session->signSinglePart = true;
+        break;
+      case CKM_MD5_RSA_PKCS:
+        hashFunc = new Botan::EMSA3(new Botan::MD5);
+        break;
+      case CKM_RIPEMD160_RSA_PKCS:
+        hashFunc = new Botan::EMSA3(new Botan::RIPEMD_160);
+        break;
+      case CKM_SHA1_RSA_PKCS:
+        hashFunc = new Botan::EMSA3(new Botan::SHA_160);
+        break;
+      case CKM_SHA256_RSA_PKCS:
+        hashFunc = new Botan::EMSA3(new Botan::SHA_256);
+        break;
+      case CKM_SHA384_RSA_PKCS:
+        hashFunc = new Botan::EMSA3(new Botan::SHA_384);
+        break;
+      case CKM_SHA512_RSA_PKCS:
+        hashFunc = new Botan::EMSA3(new Botan::SHA_512);
+        break;
+      default:
+        DEBUG_MSG("C_SignInit", "The selected mechanism is not supported");
+        return CKR_MECHANISM_INVALID;
+        break;
+    }
+
+    if(hashFunc == NULL_PTR) {
+      ERROR_MSG("C_SignInit", "Could not create the hash function");
+      return CKR_DEVICE_MEMORY;
+    }
 #else
-  session->signSize = (cryptoKey->max_input_bits() + 8) / 8;
-  session->pkSigner = new PK_Signer(*dynamic_cast<Private_Key*>(cryptoKey), emsa);
+    std::string emsa;
+
+    // Selects the correct padding and hash algorithm.
+    switch(pMechanism->mechanism) {
+      case CKM_RSA_PKCS:
+        emsa = "EMSA3(Raw)";
+        session->signSinglePart = true;
+        break;
+      case CKM_RSA_X_509:
+        emsa = "Raw";
+        session->signSinglePart = true;
+        break;
+      case CKM_MD5_RSA_PKCS:
+        emsa = "EMSA3(MD5)";
+        break;
+      case CKM_RIPEMD160_RSA_PKCS:
+        emsa = "EMSA3(RIPEMD-160)";
+        break;
+      case CKM_SHA1_RSA_PKCS:
+        emsa = "EMSA3(SHA-160)";
+        break;
+      case CKM_SHA256_RSA_PKCS:
+        emsa = "EMSA3(SHA-256)";
+        break;
+      case CKM_SHA384_RSA_PKCS:
+        emsa = "EMSA3(SHA-384)";
+        break;
+      case CKM_SHA512_RSA_PKCS:
+        emsa = "EMSA3(SHA-512)";
+        break;
+      default:
+        DEBUG_MSG("C_SignInit", "The selected mechanism is not supported");
+        return CKR_MECHANISM_INVALID;
+        break;
+    }
 #endif
 
-  if(!session->pkSigner) {
-    DEBUG_MSG("C_SignInit", "Could not create the signing function");
-    return CKR_DEVICE_MEMORY;
+    // Creates the signer with given key and mechanism.
+    try {
+#ifdef BOTAN_PRE_1_9_4_FIX
+      Botan::PK_Signing_Key *signKey = dynamic_cast<Botan::PK_Signing_Key*>(cryptoKey);
+      session->signSize = (cryptoKey->max_input_bits() + 8) / 8;
+      session->pkSigner = new Botan::PK_Signer(*signKey, &*hashFunc);
+#else
+      session->signSize = (cryptoKey->max_input_bits() + 8) / 8;
+      session->pkSigner = new Botan::PK_Signer(*dynamic_cast<Botan::Private_Key*>(cryptoKey), emsa);
+#endif
+    }
+    catch(std::exception& e) {
+      char errorMsg[1024];
+      snprintf(errorMsg, sizeof(errorMsg), "Could not create the signing function: %s", e.what());
+      ERROR_MSG("C_SignInit", errorMsg);
+      return CKR_GENERAL_ERROR;
+    }
+
+    if(!session->pkSigner) {
+      ERROR_MSG("C_SignInit", "Could not create the signing function");
+      return CKR_DEVICE_MEMORY;
+    }
+
+    session->signMech = pMechanism->mechanism;
+    session->signKey = hKey;
+
+#ifndef BOTAN_NO_PK_SIGNER_REUSE
   }
+#endif
 
   session->signInitialized = true;
 
@@ -1265,17 +1741,30 @@ CK_RV C_Sign(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pData, CK_ULONG ulDataLen,
     return CKR_ARGUMENTS_BAD;
   }
 
-  // Sign 
-  SecureVector<byte> signResult = session->pkSigner->sign_message(pData, ulDataLen, *session->rng);
+  // Sign
+  Botan::SecureVector<Botan::byte> signResult;
+  try {
+    signResult = session->pkSigner->sign_message(pData, ulDataLen, *session->rng);
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not sign the data: %s", e.what());
+    ERROR_MSG("C_Sign", errorMsg);
+
+    // Finalizing
+    session->signSize = 0;
+    delete session->pkSigner;
+    session->pkSigner = NULL_PTR;
+    session->signInitialized = false;
+
+    return CKR_GENERAL_ERROR;
+  }
 
   // Returns the result
   memcpy(pSignature, signResult.begin(), session->signSize);
   *pulSignatureLen = session->signSize;
 
   // Finalizing
-  session->signSize = 0;
-  delete session->pkSigner;
-  session->pkSigner = NULL_PTR;
   session->signInitialized = false;
 
   DEBUG_MSG("C_Sign", "OK");
@@ -1314,7 +1803,22 @@ CK_RV C_SignUpdate(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pPart, CK_ULONG ulPar
   }
 
   // Buffer
-  session->pkSigner->update(pPart, ulPartLen);
+  try {
+    session->pkSigner->update(pPart, ulPartLen);
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not buffer the data: %s", e.what());
+    ERROR_MSG("C_SignUpdate", errorMsg);
+
+    // Finalizing
+    session->signSize = 0;
+    delete session->pkSigner;
+    session->pkSigner = NULL_PTR;
+    session->signInitialized = false;
+
+    return CKR_GENERAL_ERROR;
+  }
 
   DEBUG_MSG("C_SignUpdate", "OK");
   return CKR_OK;
@@ -1366,16 +1870,29 @@ CK_RV C_SignFinal(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pSignature, CK_ULONG_P
   }
 
   // Sign
-  SecureVector<byte> signResult = session->pkSigner->signature(*session->rng);
+  Botan::SecureVector<Botan::byte> signResult;
+  try {
+    signResult = session->pkSigner->signature(*session->rng);
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not sign the data: %s", e.what());
+    ERROR_MSG("C_SignFinal", errorMsg);
+
+    // Finalizing
+    session->signSize = 0;
+    delete session->pkSigner;
+    session->pkSigner = NULL_PTR;
+    session->signInitialized = false;
+
+    return CKR_GENERAL_ERROR;
+  }
 
   // Returns the result
   memcpy(pSignature, signResult.begin(), session->signSize);
   *pulSignatureLen = session->signSize;
 
   // Finalizing
-  session->signSize = 0;
-  delete session->pkSigner;
-  session->pkSigner = NULL_PTR;
   session->signInitialized = false;
 
   DEBUG_MSG("C_SignFinal", "OK");
@@ -1444,35 +1961,35 @@ CK_RV C_VerifyInit(CK_SESSION_HANDLE hSession, CK_MECHANISM_PTR pMechanism, CK_O
 
   session->verifySinglePart = false;
 #ifdef BOTAN_PRE_1_9_4_FIX
-  EMSA *hashFunc = NULL_PTR;
+  Botan::EMSA *hashFunc = NULL_PTR;
 
   // Selects the correct padding and hash algorithm.
   switch(pMechanism->mechanism) {
     case CKM_RSA_PKCS:
-      hashFunc = new EMSA3_Raw();
+      hashFunc = new Botan::EMSA3_Raw();
       session->verifySinglePart = true;
       break;
     case CKM_RSA_X_509:
-      hashFunc = new EMSA_Raw();
+      hashFunc = new Botan::EMSA_Raw();
       session->verifySinglePart = true;
       break;
     case CKM_MD5_RSA_PKCS:
-      hashFunc = new EMSA3(new MD5);
+      hashFunc = new Botan::EMSA3(new Botan::MD5);
       break;
     case CKM_RIPEMD160_RSA_PKCS:
-      hashFunc = new EMSA3(new RIPEMD_160);
+      hashFunc = new Botan::EMSA3(new Botan::RIPEMD_160);
       break;
     case CKM_SHA1_RSA_PKCS:
-      hashFunc = new EMSA3(new SHA_160);
+      hashFunc = new Botan::EMSA3(new Botan::SHA_160);
       break;
     case CKM_SHA256_RSA_PKCS:
-      hashFunc = new EMSA3(new SHA_256);
+      hashFunc = new Botan::EMSA3(new Botan::SHA_256);
       break;
     case CKM_SHA384_RSA_PKCS:
-      hashFunc = new EMSA3(new SHA_384);
+      hashFunc = new Botan::EMSA3(new Botan::SHA_384);
       break;
     case CKM_SHA512_RSA_PKCS:
-      hashFunc = new EMSA3(new SHA_512);
+      hashFunc = new Botan::EMSA3(new Botan::SHA_512);
       break;
     default:
       DEBUG_MSG("C_VerifyInit", "The selected mechanism is not supported");
@@ -1523,24 +2040,32 @@ CK_RV C_VerifyInit(CK_SESSION_HANDLE hSession, CK_MECHANISM_PTR pMechanism, CK_O
 #endif
 
   // Get the key from the session key store.
-  Public_Key *cryptoKey = session->getKey(hKey);
+  Botan::Public_Key *cryptoKey = session->getKey(hKey);
   if(cryptoKey == NULL_PTR) {
     DEBUG_MSG("C_VerifyInit", "Could not load the crypto key");
     return CKR_GENERAL_ERROR;
   }
 
   // Creates the verifier with given key and mechanism
+  try {
 #ifdef BOTAN_PRE_1_9_4_FIX
-  PK_Verifying_with_MR_Key *verifyKey = dynamic_cast<PK_Verifying_with_MR_Key*>(cryptoKey);
-  session->verifySize = (cryptoKey->max_input_bits() + 8) / 8;
-  session->pkVerifier = new PK_Verifier_with_MR(*verifyKey, &*hashFunc);
+    Botan::PK_Verifying_with_MR_Key *verifyKey = dynamic_cast<Botan::PK_Verifying_with_MR_Key*>(cryptoKey);
+    session->verifySize = (cryptoKey->max_input_bits() + 8) / 8;
+    session->pkVerifier = new Botan::PK_Verifier_with_MR(*verifyKey, &*hashFunc);
 #else
-  session->verifySize = (cryptoKey->max_input_bits() + 8) / 8;
-  session->pkVerifier = new PK_Verifier(*cryptoKey, emsa);
+    session->verifySize = (cryptoKey->max_input_bits() + 8) / 8;
+    session->pkVerifier = new Botan::PK_Verifier(*cryptoKey, emsa);
 #endif
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not create the verifying function: %s", e.what());
+    ERROR_MSG("C_VerifyInit", errorMsg);
+    return CKR_GENERAL_ERROR;
+  }
 
   if(!session->pkVerifier) {
-    DEBUG_MSG("C_VerifyInit", "Could not create the verifying function");
+    ERROR_MSG("C_VerifyInit", "Could not create the verifying function");
     return CKR_DEVICE_MEMORY;
   }
 
@@ -1592,7 +2117,22 @@ CK_RV C_Verify(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pData, CK_ULONG ulDataLen
   }
 
   // Verify
-  bool verResult = session->pkVerifier->check_signature(pSignature, ulSignatureLen);
+  bool verResult;
+  try {
+    verResult = session->pkVerifier->check_signature(pSignature, ulSignatureLen);
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not check the signature: %s", e.what());
+    ERROR_MSG("C_Verify", errorMsg);
+
+    // Finalizing
+    delete session->pkVerifier;
+    session->pkVerifier = NULL_PTR;
+    session->verifyInitialized = false;
+
+    return CKR_GENERAL_ERROR;
+  }
 
   // Finalizing
   delete session->pkVerifier;
@@ -1641,7 +2181,21 @@ CK_RV C_VerifyUpdate(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pPart, CK_ULONG ulP
   }
 
   // Add data
-  session->pkVerifier->update(pPart, ulPartLen);
+  try {
+    session->pkVerifier->update(pPart, ulPartLen);
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not buffer the data: %s", e.what());
+    ERROR_MSG("C_VerifyUpdate", errorMsg);
+
+    // Finalizing
+    delete session->pkVerifier;
+    session->pkVerifier = NULL_PTR;
+    session->verifyInitialized = false;
+
+    return CKR_GENERAL_ERROR;
+  }
 
   DEBUG_MSG("C_VerifyUpdate", "OK");
   return CKR_OK;
@@ -1690,7 +2244,22 @@ CK_RV C_VerifyFinal(CK_SESSION_HANDLE hSession, CK_BYTE_PTR pSignature, CK_ULONG
   }
 
   // Verify
-  bool verResult = session->pkVerifier->check_signature(pSignature, ulSignatureLen);
+  bool verResult;
+  try {
+    verResult = session->pkVerifier->check_signature(pSignature, ulSignatureLen);
+  }
+  catch(std::exception& e) {
+    char errorMsg[1024];
+    snprintf(errorMsg, sizeof(errorMsg), "Could not check the signature: %s", e.what());
+    ERROR_MSG("C_VerifyFinal", errorMsg);
+
+    // Finalizing
+    delete session->pkVerifier;
+    session->pkVerifier = NULL_PTR;
+    session->verifyInitialized = false;
+
+    return CKR_GENERAL_ERROR;
+  }
 
   // Finalizing
   delete session->pkVerifier;
@@ -1937,7 +2506,7 @@ CK_RV rsaKeyGen(SoftSession *session, CK_ATTRIBUTE_PTR pPublicKeyTemplate,
 
   CK_ULONG *modulusBits = NULL_PTR;
   // Defaults to an exponent with e = 65537
-  BigInt *exponent = new Botan::BigInt("65537");;
+  Botan::BigInt *exponent = new Botan::BigInt("65537");;
   CHECK_DEBUG_RETURN(exponent == NULL_PTR, "C_GenerateKeyPair", "Could not allocate memory", CKR_HOST_MEMORY);
 
   // Extract desired key information
@@ -1954,7 +2523,7 @@ CK_RV rsaKeyGen(SoftSession *session, CK_ATTRIBUTE_PTR pPublicKeyTemplate,
         break;
       case CKA_PUBLIC_EXPONENT:
         delete exponent;
-        exponent = new Botan::BigInt((byte*)pPublicKeyTemplate[i].pValue,(u32bit)pPublicKeyTemplate[i].ulValueLen);
+        exponent = new Botan::BigInt((Botan::byte*)pPublicKeyTemplate[i].pValue,(Botan::u32bit)pPublicKeyTemplate[i].ulValueLen);
         break;
       default:
         break;
@@ -1970,9 +2539,9 @@ CK_RV rsaKeyGen(SoftSession *session, CK_ATTRIBUTE_PTR pPublicKeyTemplate,
   }
 
   // Generate the key
-  RSA_PrivateKey *rsaKey = NULL_PTR;
+  Botan::RSA_PrivateKey *rsaKey = NULL_PTR;
   try {
-    rsaKey = new RSA_PrivateKey(*session->rng, (u32bit)*modulusBits, exponent->to_u32bit());
+    rsaKey = new Botan::RSA_PrivateKey(*session->rng, (Botan::u32bit)*modulusBits, BotanCompat::to_u32bit(*exponent));
     delete exponent;
   }
   catch(...) {
@@ -1983,7 +2552,7 @@ CK_RV rsaKeyGen(SoftSession *session, CK_ATTRIBUTE_PTR pPublicKeyTemplate,
   CHECK_DEBUG_RETURN(rsaKey == NULL_PTR, "C_GenerateKeyPair", "Could not allocate memory", CKR_HOST_MEMORY);
 
   // Add the private key to the database.
-  CK_OBJECT_HANDLE privRef = session->db->addRSAKeyPriv(rsaKey, pPrivateKeyTemplate, ulPrivateKeyAttributeCount);
+  CK_OBJECT_HANDLE privRef = session->db->addRSAKeyPriv(session->getSessionState(), rsaKey, pPrivateKeyTemplate, ulPrivateKeyAttributeCount);
 
   if(privRef == 0) {
     delete rsaKey;
@@ -1993,7 +2562,7 @@ CK_RV rsaKeyGen(SoftSession *session, CK_ATTRIBUTE_PTR pPublicKeyTemplate,
   }
 
   // Add the public key to the database.
-  CK_OBJECT_HANDLE pubRef = session->db->addRSAKeyPub(rsaKey, pPublicKeyTemplate, ulPublicKeyAttributeCount);
+  CK_OBJECT_HANDLE pubRef = session->db->addRSAKeyPub(session->getSessionState(), rsaKey, pPublicKeyTemplate, ulPublicKeyAttributeCount);
   delete rsaKey;
 
   if(pubRef == 0) {
