@@ -1,4 +1,4 @@
-/* $Id: tokenhandling.cpp 3904 2010-09-09 09:07:59Z rb $ */
+/* $Id: tokenhandling.cpp 4674 2011-04-01 09:40:16Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -79,7 +79,7 @@ static char sqlDeleteTrigger[] =
   "END;";
 
 static char sqlCreateIndexAttributes[] =
-  "CREATE INDEX idxObject ON Attributes (objectID);";
+  "CREATE INDEX idxObject ON Attributes (objectID, type);";
 
 // Initialize a token
 
